<?php
// namespace administrator\components\com_jfbchat\controllers;
/**
 *
 * @package FBCHAT::MESSAGES::administrator::components::com_jfbchat
 * @subpackage controllers
 * @author 2Punti - Marco Biagioni
 * @copyright (C) 2012 - 2PUNTI SRL
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */
defined ( '_JEXEC' ) or die ( 'Restricted access' );
jimport ( 'joomla.application.component.controller' );

/**
 * User leaved messages controller responsibilities contract
 *
 * @package FBCHAT::MESSAGES::administrator::components::com_jfbchat
 * @subpackage controllers
 * @since 1.0
 */
interface ILAMessagesController {

	/**
	 * Mostra il dettaglio dell'entity
	 * 
	 * @access public
	 * @return void
	 */
	public function editEntity();
	
	/**
	 * Gestisce il save dell'edit entity apply/save
	 *
	 * @access public
	 * @return void
	 */
	public function saveEntity();
	
	/**
	 * Cancella una entity dal DB
	 *
	 * @access public
	 * @return void
	 */
	public function deleteEntity();
 
	/**
	 * Avvia il processo di esportazione records
	 *
	 * @access public
	 * @return void
	 */
	public function exportMessages();
 
	/**
	 * Modifica lo stato di worked del record per
	 * il ricontatto dell'utente 
	 * 
	 * @access public
	 * @return void
	 */
	public function workedFlag();
	
	/**
	 * Risponde all'email del richiedente con il messaggio inserito dall'agente
	 * richiedendo la serializzazione delle risposte nell'apposito db field
	 *
	 * @access public
	 * @return void
	 */
	public function responseMessage();
}	

/**
 * User leaved messages concrete implementation
 *
 * @package FBCHAT::MESSAGES::administrator::components::com_jfbchat
 * @subpackage controllers
 * @since 1.0
 */
class JfbchatControllerLAMessages extends jfbcController implements ILAMessagesController { 
	/**
	 * Setta il model state a partire dallo userstate di sessione
	 * @access protected
	 * @return void
	 */
	protected function setModelState() {
		parent::setModelState('lamessages');
		
		// User state specific
		$app = JFactory::getApplication();
		$option= JRequest::getVar('option');
		  
		$filter_order = $app->getUserStateFromRequest( "$option.lamessages.filter_order", 'filter_order', 'a.sentdate', 'cmd' );
		$fromPeriod = $app->getUserStateFromRequest( "$option.lamessages.fromperiod", 'fromperiod');
		$toPeriod = $app->getUserStateFromRequest( "$option.lamessages.toperiod", 'toperiod');
		$worked = $app->getUserStateFromRequest( "$option.lamessages.workedfilter", 'workedfilter');
		
		// Get default model
		$defaultModel = $this->getModel();
		
		// Set model state  
		$defaultModel->setState('order', $filter_order);
		$defaultModel->setState('fromPeriod', $fromPeriod);
		$defaultModel->setState('toPeriod', $toPeriod);
		$defaultModel->setState('workedfilter', $worked);
	}
	
	/**
	 * Default listEntities
	 * 
	 * @access public
	 * @return void
	 */
	public function display() {
		// Set model state 
		$this->setModelState();
		
		// Parent construction and view display
		parent::display();
	}

	/**
	 * Mostra il dettaglio dell'entity
	 * 
	 * @access public
	 * @return void
	 */
	public function editEntity() {
		$cid = JRequest::getVar('cid', array(0), 'method', 'array');
		$idEntity = (int) $cid[0];
		$model = $this->getModel();
		$record = $model->loadEntity($idEntity);
		
		$model->setState('option', JRequest::getVar('option'));
 
		$view = $this->getView();
		$view->setModel ( $model, true );
		$view->showEntity($record); 
	}
	
	/**
	 * Gestisce il save dell'edit entity apply/save
	 *
	 * @access public
	 * @return void
	 */
	public function saveEntity() {    
		$task = JRequest::getCmd('task', 'saveEntity');
		$option = JRequest::getVar('option');
		//Load della  model e bind store
		$model = $this->getModel ( );
		$table = $model->storeEntity(); 
		
		$msgPrefix = $table ? 'SUCCESS' : 'ERROR';
		$msgSufix = $task == 'lamessages.saveEntity' ? '_SAVING' : '_APPLY';
		$controllerTask = $task == 'lamessages.saveEntity' ? 'display' : 'editEntity&cid[]=' . $table->id;
		$msg = $msgPrefix . $msgSufix;

		$this->setRedirect ( "index.php?option=$option&task=lamessages.$controllerTask", JTEXT::_($msg)); 
	}
	
	/**
	 * Cancella una entity dal DB
	 *
	 * @access public
	 * @return void
	 */
	public function deleteEntity() {
		$cids = JRequest::getVar('cid', array(0), 'method', 'array');
		$option = JRequest::getVar('option');
		//Load della  model e checkin before exit
		$model = $this->getModel ( );
		$result = $model->deleteEntity($cids);
	
		$msgResult = $result ? 'SUCCESS_DELETE' : 'ERROR_DELETE';
			
		$this->setRedirect ( "index.php?option=$option&task=lamessages.display", JTEXT::_($msgResult) );
	}

	/**
	 * Avvia il processo di esportazione records
	 *
	 * @access public
	 * @return void
	 */
	public function exportMessages() {
		// Set model state
		$this->setModelState();
		// Mapping fields to load to column header
		$fieldsToLoadArray = array(	'a.name'=>JText::_('LAMESSAGE_NAME'),
									'a.email'=>JText::_('LAMESSAGE_EMAIL'),
									'a.message'=>JText::_('MESSAGE'),  
									'a.sentdate'=>JText::_('SENT'),
									'a.worked'=>JText::_('WORKED_STATE'),
									'u.name AS username_logged'=>JText::_('USERID'));
		$fieldsFunctionTransformation = array();
	
		$model = $this->getModel();
		$data = $model->exportMessages($fieldsToLoadArray, $fieldsFunctionTransformation);
	
		if(!$data) {
			$this->setRedirect('index.php?option=com_jfbchat&task=lamessages.display', JText::_('NODATA_EXPORT'));
			return false;
		}
	
		// Get view
		$view = $this->getView();
		$view->sendCSVMessages($data, $fieldsFunctionTransformation);
	}
	
	/**
	 * @access public
	 */
	public function workedFlag() {
		$cid = JRequest::getVar('cid', array(0), 'method', 'array');
		$idEntity = (int) $cid[0];
		$model = $this->getModel();
		$result = $model->changeWorkedState($idEntity, $this->task);
		$msg = null;
		if ($result) {
			$msg = JText::_('LAMESSAGE_STATE_CHANGED');
		}
		$this->setRedirect( 'index.php?option=com_jfbchat&task=lamessages.display', $msg);
	}
	
	/**
	 * Risponde all'email del richiedente con il messaggio inserito dall'agente
	 * richiedendo la serializzazione delle risposte nell'apposito db field
	 *
	 * @access public
	 * @return void
	 */
	public function responseMessage() {
		$task = JRequest::getCmd('task', 'responseMessage');
		$option = JRequest::getVar('option');
		$responseSubject = JRequest::getString('email_subject');
		$responseText = JRequest::getString('response', '', 'POST', JREQUEST_ALLOWHTML);
		$idEntity = JRequest::getInt('id');
		
		// Response text vuota validazione lato server con return false
		if(!trim($responseText) || !trim($responseSubject)) {
			$msgEsito = 'VALIDATION_ERROR';
			$controllerTask = 'editEntity&cid[]=' . $idEntity; 
			$this->setRedirect ( "index.php?option=$option&task=lamessages.$controllerTask", JTEXT::_($msgEsito));
			return false;
		}
		 
		$model = $this->getModel();
		$table = $model->sendResponseStore($idEntity, $responseSubject, $responseText);
		
		$msgEsito = $table ? 'SUCCESS_SEND_MESSAGE' : 'ERROR_SEND_MESSAGE';
		$controllerTask = 'editEntity&cid[]=' . $idEntity; 
		
		$this->setRedirect ( "index.php?option=$option&task=lamessages.$controllerTask", JTEXT::_($msgEsito));
	}
	
	/**
	 * Class constructor
	 * @return Object&
	 */
	public function __construct($config = array()){
		parent::__construct($config);
	
		// Registering alias task
		$this->registerTask('applyEntity', 'saveEntity');
		$this->registerTask('workedFlagOff', 'workedFlag');
		$this->registerTask('workedFlagOn', 'workedFlag');
	}
}
?>