<?php
// namespace administrator\components\com_jfbchat\controllers;
/** 
 * @package FBCHAT::SUGGESTIONS::administrator::components::com_jfbchat
 * @subpackage controllers
 * @author 2Punti - Marco Biagioni
 * @copyright (C) 2012 - 2PUNTI SRL
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */
defined ( '_JEXEC' ) or die ( 'Restricted access' );
jimport ( 'joomla.application.component.controller' );

/**
 * Suggestions controller responsibilities contract
 *
 * @package FBCHAT::SUGGESTIONS::administrator::components::com_jfbchat
 * @subpackage controllers
 * @since 1.0
 */
interface ISuggestionsController { 
	/**
	 * Mostra il dettaglio dell'entity
	 * 
	 * @access public
	 * @return void
	 */
	public function editEntity();

	/**
	 * Gestisce il save dell'edit entity apply/save
	 *
	 * @access public
	 * @return void
	 */
	public function saveEntity();
	
	/**
	 * Cancella una entity dal DB
	 *
	 * @access public
	 * @return void
	 */
	public function deleteEntity();

	/**
	 * Gestisce il cancel edit e unlock del record checked out
	 *
	 * @access public
	 * @return void
	 */
	public function cancelEntity();

	/**
	 * Imposta la suggestion come il messaggio predefinito da mostrarsi nel fumetto
	 * di frontend autosuggestion
	 *
	 * @access public
	 * @return void
	 */
	public function autoInviteFlag();
	
	/**
	 * Cambia lo stato di publicazione del suggerimento
	 *
	 * @access public
	 * @return void
	 */
	public function publishEntity();
}

/**
 * Suggestions controller concrete implementation
 *
 * @package FBCHAT::SUGGESTIONS::administrator::components::com_jfbchat
 * @subpackage controllers
 * @since 1.0
 */
class JfbchatControllerSuggestions extends jfbcController implements ISuggestionsController { 
	/**
	 * Default listEntities
	 * 
	 * @access public
	 * @return void
	 */
	public function display() {
		// Set model state 
		$this->setModelState('suggestions');
		
		// Parent construction and view display
		parent::display();
	}

	/**
	 * Mostra il dettaglio dell'entity
	 * 
	 * @access public
	 * @return void
	 */
	public function editEntity() { 
		JRequest::setVar('hidemainmenu', 1);  
		$cid = JRequest::getVar('cid', array(0), 'method', 'array');
		$idEntity = (int) $cid[0];
		$user = JFactory::getUser();
		$model = $this->getModel();
		$record = $model->loadEntity($idEntity);
		
		$model->setState('option', JRequest::getVar('option'));
		
		// Check out del record
		if ($record->checked_out && $record->checked_out != $user->id) {
			$this->setRedirect ( 'index.php?option=com_jfbchat&task=suggestions.display', JTEXT::_('CHECKEDOUT_RECORD'));
			return false;
		}
		
		if ($record->id) {
			$record->checkout ( $user->id );
		}
		
		$view = $this->getView();
		$view->setModel ( $model, true );
		$view->editEntity($record);
		
	}

	/**
	 * Gestisce il save dell'edit entity apply/save
	 *
	 * @access public
	 * @return void
	 */
	public function saveEntity() {
		$task = JRequest::getCmd('task', 'saveEntity');
		$option = JRequest::getVar('option');
		//Load della  model e bind store
		$model = $this->getModel ( );
		$table = $model->storeEntity(); 
		
		$msgPrefix = $table ? 'SUCCESS' : 'ERROR';
		$msgSufix = $task == 'suggestions.saveEntity' ? '_SAVING' : '_APPLY';
		$controllerTask = $task == 'suggestions.saveEntity' ? 'display' : 'editEntity&cid[]=' . $table->id;
		$msg = $msgPrefix . $msgSufix;

		$this->setRedirect ( "index.php?option=$option&task=suggestions.$controllerTask", JTEXT::_($msg)); 
	}

	/**
	 * Cancella una entity dal DB
	 *
	 * @access public
	 * @return void
	 */
	public function deleteEntity() {
		$cids = JRequest::getVar('cid', array(0), 'method', 'array'); 
		$option = JRequest::getVar('option');
		//Load della  model e checkin before exit
		$model = $this->getModel ( );
		$result = $model->deleteEntity($cids);
		
		$msgResult = $result ? 'SUCCESS_DELETE' : 'ERROR_DELETE';  
			
		$this->setRedirect ( "index.php?option=$option&task=suggestions.display", JTEXT::_($msgResult) );
	}
	
	/**
	 * Gestisce il cancel edit e unlock del record checked out
	 *
	 * @access public
	 * @return void
	 */
	public function cancelEntity() { 
		$id = JRequest::getVar('id');
		$option = JRequest::getVar('option');
		//Load della  model e checkin before exit
		$model = $this->getModel ( );
		$model->cancelEntity($id); 
		 
		$this->setRedirect ( "index.php?option=$option&task=suggestions.display", JTEXT::_('CANCELED_OPERATION') );
	}

	/**
	 * Imposta la suggestion come il messaggio predefinito da mostrarsi nel fumetto
	 * di frontend autosuggestion
	 *
	 * @access public
	 * @return void
	 */
	public function autoInviteFlag() {
		$cid = JRequest::getVar('cid', array(0), 'method', 'array');
		$idEntity = (int) $cid[0];
		$model = $this->getModel();
		$result = $model->setAutoInviteFlag($idEntity, $this->task);
		$msg = null;
		if ($result) {
			$msg = JText::_('SUGGESTION_STATE_CHANGED');
		}
		$this->setRedirect( 'index.php?option=com_jfbchat&task=suggestions.display', $msg);
	}
	
	/**
	 * Cambia lo stato di publicazione del suggerimento 
	 *
	 * @access public
	 * @return void
	 */
	public function publishEntity() {
		$cid = JRequest::getVar('cid', array(0), 'method', 'array');
		$idEntity = (int) $cid[0];
		$model = $this->getModel();
		$result = $model->publishEntity($idEntity, $this->task);
		$msg = null;
		if ($result) {
			$msg = JText::_('SUGGESTION_STATE_CHANGED');
		}
		$this->setRedirect( 'index.php?option=com_jfbchat&task=suggestions.display', $msg);
	}
	
	/**
	 * Class constructor
	 * @return Object&
	 */
	public function __construct($config = array()){
		parent::__construct($config);
	
		// Registering alias task
		$this->registerTask('applyEntity', 'saveEntity');
		$this->registerTask('publish', 'publishEntity');
		$this->registerTask('unpublish', 'publishEntity');
		$this->registerTask('defaultinvite', 'autoInviteFlag');
		$this->registerTask('nodefaultinvite', 'autoInviteFlag');
	}
}
?>