<?php
// namespace administrator\components\com_jfbchat\models;
/**
 *
 * @package FBCHAT::AGENTS::administrator::components::com_jfbchat
 * @subpackage models
 * @author 2Punti - Marco Biagioni
 * @copyright (C) 2012 - 2PUNTI SRL
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */
defined ( '_JEXEC' ) or die ( 'Restricted access' ); 

/**
 * Agents model responsibilities
 *
 * @package FBCHAT::AGENTS::administrator::components::com_jfbchat
 * @subpackage models
 * @since 1.0
 */
interface IAgentsModel { 
	/**
	 * Inserisce o elimina il record nella #__fbchat_agents
	 * @access public
	 * @param int $idEntity
	 * @param string $state
	 * @return boolean
	 */
	public function changeAgentState($idEntity, $state);

	/**
	 * Main get data method
	 * @access public
	 * @return Object[]
	 */
	public function getData();

	/**
	 * Counter result set
	 * @access public
	 * @return int
	 */
	public function getTotal();
	
	/**
	 * Restituisce le select list usate dalla view per l'interfaccia
	 * @access public
	 * @return array
	 */
	public function getLists();
}


/**
 * Agents model concrete implementation
 *
 * @package FBCHAT::CONFIG::administrator::components::com_jfbchat
 * @subpackage models
 * @since 1.0
 */
class JfbchatModelAgents extends JModelLegacy implements IAgentsModel {
	/**
	 * Records query result set
	 * @access private
	 * @var Object[]
	 */
	private $records;

	/**
	 * Costruzione list entities query
	 * @access private
	 * @return string
	 */
	private function buildListQuery() {
		// WHERE
		$where = array();
		$whereString = null;
		// Filtro per gruppi
		if($this->state->get('filter_type')) {
			$where[] = "groups.id = " . (int)$this->state->get('filter_type');
		}
		
		//Filtro testo
		if($this->state->get('searchword')) {
			$where[] = "a.name LIKE '%" . $this->state->get('searchword') . "%'";
		}
		
		if (count($where)) {
			$whereString = "\n WHERE " . implode ("\n AND ", $where);
		}
		
		// ORDERBY
		if($this->state->get('order')) {
			$orderString = "\n ORDER BY " . $this->state->get('order') . " ";
		}
		
		//Filtro testo
		if($this->state->get('order_dir')) {
			$orderString .= $this->state->get('order_dir');
		}
		
		
		$query = "SELECT a.*, groups.title AS groupname, ag.userid AS agent"
				. "\n FROM #__users AS a"
				. "\n INNER JOIN #__user_usergroup_map AS map ON a.id = map.user_id"
				. "\n INNER JOIN #__usergroups AS groups ON groups.id = map.group_id"
				. "\n LEFT JOIN #__fbchat_agents AS ag ON ag.userid = a.id"
				. $whereString 
				. "\n GROUP BY a.id"
				. $orderString;
		return $query;
	}

	/**
	 * Inserisce o elimina il record nella #__fbchat_agents
	 * @access public
	 * @param int $idEntity
	 * @param string $state
	 * @return boolean
	 */
	public function changeAgentState($idEntity, $state) {
		switch($state){
			case 'makeagent':
				$query = "INSERT INTO #__fbchat_agents (". $this->_db->quoteName('userid'). ")".
						 "\n VALUES('$idEntity')";
				break;
				
			case 'destroyagent':
				$query = "DELETE FROM #__fbchat_agents WHERE ". $this->_db->quoteName('userid').
				"\n = $idEntity";
				break;
		}
		$this->_db->setQuery($query);
		if(!$this->_db->execute()){
			JError::raiseNotice ( 400, $this->_db->getErrorMsg () );
			return false;
		}
		return true;
	}

	/**
	 * Main get data method
	 * @access public
	 * @return Object[]
	 */
	public function getData() {
		// Build query 
		$query = $this->buildListQuery();
		$this->_db->setQuery( $query, $this->getState('limitstart'), $this->getState('limit') );
		$result = $this->_db->loadObjectList();

		return $result;
	}

	/**
	 * Counter result set
	 * @access public
	 * @return int
	 */
	public function getTotal() {
		// Build query 
		$query = $this->buildListQuery();
		$this->_db->setQuery( $query);
		$result = count( $this->_db->loadColumn() );

		return $result;
	}
	
	/**
	 * Restituisce le select list usate dalla view per l'interfaccia
	 * @access public
	 * @return array
	 */
	public function getLists() { 
		$lists = array();
		$this->_db->setQuery(
			'SELECT a.id AS value, a.title AS text, COUNT(DISTINCT b.id) AS level' .
			' FROM #__usergroups AS a' .
			' LEFT JOIN '.$this->_db->quoteName('#__usergroups').' AS b ON a.lft > b.lft AND a.rgt < b.rgt' .
			' GROUP BY a.id, a.title, a.lft, a.rgt' .
			' ORDER BY a.lft ASC'
		);
		$options = $this->_db->loadObjectList(); 
		// Check for a database error.
		if ($this->_db->getErrorNum()) {
			JError::raiseNotice(500, $this->_db->getErrorMsg());
			return null;
		} 
		$default = JHTML::_('select.option', '', JText::_('ALL_GROUPS'));
		array_unshift($options, $default);
		foreach ($options as &$option) {
			if(isset($option->level)) {
				$option->text = str_repeat('- ', $option->level).$option->text; 
			}
		}
		$lists['type'] 	= JHTML::_('select.genericlist', $options, 'filter_type', 'class="inputbox" size="1" onchange="document.adminForm.submit( );"', 'value', 'text', $this->state->get('filter_type'));
		 
		return $lists;
	}
}
?>