<?php
// namespace administrator\components\com_jfbchat\models;
/**
 *
 * @package FBCHAT::MESSAGES::administrator::components::com_jfbchat
 * @subpackage models
 * @author 2Punti - Marco Biagioni
 * @copyright (C) 2012 - 2PUNTI SRL
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */
defined ( '_JEXEC' ) or die ( 'Restricted access' ); 

/**
 * User leaved messages model responsibilities contract
 *
 * @package FBCHAT::MESSAGES::administrator::components::com_jfbchat
 * @subpackage models
 * @since 1.0
 */
interface ILAMessagesModel { 
	/**
	 * Main get data method
	 * @access public
	 * @return Object[]
	 */
	public function getData();

	/**
	 * Counter result set
	 * @access public
	 * @return int
	 */
	public function getTotal();

	/**
	 * Restituisce le select list usate dalla view per l'interfaccia
	 * @access public
	 * @return array
	 */
	public function getLists();
	
	/**
	 * Effettua il load dell'entity singola da orm table
	 * @access public
	 * @param int $id
	 * @return Object&
	 */
	public function loadEntity($id);
	
	/**
	 * Storing record tramite $table
	 * @access public
	 * @return mixed
	 */
	public function storeEntity();
	 
	/**
	 * Modifica lo stato lavorato del record
	 * @param int $id
	 * @param string $state
	 * @access public
	 * @return boolean
	 */
	public function changeWorkedState($id, $state);
	
	/**
	 * Esplica la funzione di esportazione della lista messaggi
	 * in formato CSV per i record estratti dai filtri userstate attivi
	 * @access public
	 * @param array $fieldsToLoadArray
	 * @param array& $fieldsFunctionTransformation
	 * @return Object[]&
	 */
	public function exportMessages($fieldsToLoadArray, &$fieldsFunctionTransformation);
}

/**
 * User leaved messages model responsibilities
 *
 * @package FBCHAT::MESSAGES::administrator::components::com_jfbchat
 * @subpackage models
 * @since 1.0
 */
class JfbchatModelLAMessages extends JModelLegacy implements ILAMessagesModel {
	/**
	 * Dataset records
	 * @var Object[]
	 * @access private
	 */
	private $records;
	
	/**
	 * Component configuration pointer
	 *
	 * @access private
	 * @var Object&
	 */
	private $config;
	  
	/**
	 * Restituisce la query string costruita per ottenere il wrapped set richiesto in base
	 * allo userstate, opzionalmente seleziona i campi richiesti
	 * 
	 * @access private
	 * @return string
	 */
	private function buildListQuery($fields = 'a.*') {
		// WHERE
		$where = array();
		$whereString = null;
				
		//Filtro testo 
		if($this->state->get('searchword')) {
			$where[] = "\n (a.name LIKE " .
					$this->_db->Quote('%' . $this->state->get('searchword') . '%') .
					"\n OR a.email LIKE " .
					$this->_db->Quote('%' . $this->state->get('searchword'). '%') . ")";
		}
		
		//Filtro periodo
		if($this->state->get('fromPeriod')) {
			$where[] = "\n a.sentdate >= " . $this->_db->Quote($this->state->get('fromPeriod'));
		}
		
		if($this->state->get('toPeriod')) {
			$where[] = "\n a.sentdate <= " . $this->_db->Quote($this->state->get('toPeriod'));
		}
		
		if((int)$this->state->get('workedfilter') == 1) {
			$where[] = "\n a.worked = 1";
		} elseif((int)$this->state->get('workedfilter') == -1) {
			$where[] = "\n (a.worked != 1 OR ISNULL(a.worked))";
		}
		  
		if (count($where)) {
			$whereString = "\n WHERE " . implode ("\n AND ", $where);
		}
		
		// ORDERBY
		if($this->state->get('order')) {
			$orderString = "\n ORDER BY " . $this->state->get('order') . " ";
		}
		
		//Filtro testo
		if($this->state->get('order_dir')) {
			$orderString .= $this->state->get('order_dir');
		}
		
		$query = "SELECT $fields, u.name AS username_logged"
				. "\n FROM #__fbchat_lamessages AS a"
				. "\n LEFT JOIN #__users AS u"
				. "\n ON a.userid = u.id"
				. $whereString 
				. $orderString;
		return $query;
	}

	/**
	 * Effettua l'invio della mail di risposta al customer
	 * 
	 * @param string $recipient
	 * @param string $subject
	 * @param string $text
	 * @access private
	 * @return Object&
	 */
	private function &sendEmail($recipient, $subject, $text) {
		$mailer = & JFactory::getMailer(); 
		
		// Build e-mail message format
		$mailer->setSender(array($this->config->get('mailfrom'), $this->config->get('fromname')));
		$mailer->setSubject($subject);
		$mailer->setBody($text);
		$mailer->IsHTML(true);
		
		// Add recipient
		$mailer->addRecipient($recipient);
		  
		// Send the Mail
		$rs	= $mailer->Send();
		
		// Check for an error
		return $rs;
	}
	
	/**
	 * @access public
	 */
	public function getData() {
		// Build query 
		$query = $this->buildListQuery();
		$this->_db->setQuery( $query, $this->getState('limitstart'), $this->getState('limit') );
		$result = $this->_db->loadObjectList();

		return $result;
	}

	/**
	 * @access public
	 */
	public function getTotal() {
		// Build query 
		$query = $this->buildListQuery();
		$this->_db->setQuery( $query);
		$result = count( $this->_db->loadColumn() );

		return $result;
	}

	/**
	 * Restituisce le select list usate dalla view per l'interfaccia
	 * @access public
	 * @return array
	 */
	public function getLists() {
		$lists = array();
			
		$types[] = JHTML::_('select.option',  '', '- '. JText::_( 'MESSAGE_TYPE' ) .' -' );
		$types[] = JHTML::_('select.option', '-1', JText::_( 'NOTWORKED_MESSAGE' ) );
		$types[] = JHTML::_('select.option', '1', JText::_( 'WORKED_MESSAGE' ) );
			
		$lists['type'] 	= JHTML::_('select.genericlist', $types, 'workedfilter', 'class="inputbox" size="1" onchange="document.adminForm.task.value=\'lamessages.display\';document.adminForm.submit( );"', 'value', 'text', $this->state->get('workedfilter'));
			
		return $lists;
	}
	
	/**
	 * Effettua il load dell'entity singola da orm table
	 * @access public
	 * @param int $id
	 * @return Object&
	 */
	public function loadEntity($id) {
		// load table record
		$table = $this->getTable('Lamessages');
		$table->load($id);
		
		// Ordinamento DESC messaggi se pi di 1
		if(count($table->responses) > 1) {
			$table->responses = array_reverse($table->responses); 
		}
		 
		if(!$table){
			JError::raiseNotice ( 400, $this->_db->getErrorMsg () );
			return false;
		}
		return $table;
	}
	
	/**
	 * Storing record tramite $table
	 * @access public
	 * @return mixed
	 */
	public function storeEntity() {
		$table = $this->getTable('Lamessages');
		try {
			if (! $table->bind ($_POST)) {
				throw new Exception($table->getError ());
			}
				
			if (! $table->check ( )) {
				throw new Exception($table->getError ());
			}
			  
			if (! $table->store (false)) {
				throw new Exception($table->getError ());
			} 
		} catch(Exception $e) {
			switch ($e->getCode()) { 
				default:
					JError::raiseNotice(100, $e->getMessage());
			}
			return false;
		}
			
		return $table;
	}
	
	/**
	 * Delete entity
	 *
	 * @param array $ids
	 * @access public
	 * @return boolean
	 */
	public function deleteEntity($ids) {
		$table = $this->getTable('Lamessages');
	
		// Ciclo su ogni entity da cancellare
		foreach ($ids as $id) {
			try {
				if (! $table->delete($id)) {
					throw new Exception($table->getError ());
				}
			} catch(Exception $e) {
				switch ($e->getCode()) {
					default:
						JError::raiseNotice(100, $e->getMessage());
				}
				return false;
			}
		}
			
		return true;
	}

	/**
	 * Modifica lo stato lavorato del record
	 * @param int $id
	 * @param string $state
	 * @access public
	 * @return boolean
	 */
	public function changeWorkedState($id, $state) {
		// Table load
		$table = $this->getTable('Lamessages');
		$table->load($id);
		switch($state){
			case 'workedFlagOff':
				$table->worked = null;
				break;
				
			case 'workedFlagOn':
				$table->worked = 1;
				break; 
		}
		 
		if(!$table->store(true)){
			JError::raiseNotice ( 400, $this->_db->getErrorMsg () );
			return false;
		}
		return true;
	}
	
	/**
	 * Esplica la funzione di esportazione della lista messaggi
	 * in formato CSV per i record estratti dai filtri userstate attivi
	 * @access public
	 * @param array $fieldsToLoadArray
	 * @param array& $fieldsFunctionTransformation
	 * @return Object[]&
	 */
	public function exportMessages($fieldsToLoadArray, &$fieldsFunctionTransformation) {
		$fieldsName = array();
		if(is_array($fieldsToLoadArray) && count($fieldsToLoadArray)) {
			$arrayIter = new ArrayIterator($fieldsToLoadArray);
			while ($arrayIter->valid()) {
				$fieldName = $arrayIter->key();
				$transformedFieldName = $arrayIter->current();
				// Assegnamento duplice name0>transformation
				$fieldsName[] = $fieldName;
				$fieldsFunctionTransformation[] = $transformedFieldName;
	
				// Increment pointer
				$arrayIter->next();
			}
		}
	
		$joinedFieldsName = implode(',', $fieldsName);
	
		// Obtain query string
		$query = $this->buildListQuery($joinedFieldsName);
		$this->_db->setQuery($query);
		$resultSet = $this->_db->loadAssocList();
	
		if(!is_array($resultSet) || !count($resultSet)) {
			return false;
		}
	
		return $resultSet;
	}
	
	/**
	 * Storing record tramite $table
	 * @param int $idEntity
	 * @param string $subjectMessage
	 * @param string $textMessage
	 * @access public
	 * @return mixed
	 */
	public function sendResponseStore($idEntity, $subjectMessage, $textMessage) {
		$table = $this->getTable('Lamessages');
		try {
			$table->load($idEntity);			 
			
			// Send mail response
			if ($rs = $this->sendEmail($table->email, $subjectMessage, $textMessage)) {
				if(is_a($rs, 'JException')) {
					throw new Exception($rs->getError()); 
				}
			}
			
			if (! $table->bind ($_POST, array(), true)) {
				throw new Exception($table->getError ());
			}
	 		// Forza messaggio worked
			$table->worked = 1;
			if (! $table->store (false)) {
				throw new Exception($table->getError ());
			}
		} catch(Exception $e) {
			switch ($e->getCode()) {
				default:
					JError::raiseNotice(100, $e->getMessage());
			}
			return false;
		}
			
		return $table;
	}
	
	/**
	 * Class contructor
	 *
	 * @access public
	 * @return Object&
	 */
	public function __construct() {
		parent::__construct();
	 
		$this->config = &JComponentHelper::getParams('com_jfbchat'); 
	}
}
?>