<?php 
// namespace administrator\components\com_jfbchat\models;
/**
 *
 * @package FBCHAT::LICENSE::administrator::components::com_jfbchat
 * @subpackage models
 * @author 2Punti - Marco Biagioni
 * @copyright (C) 2012 - 2PUNTI SRL
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */
defined ( '_JEXEC' ) or die ( 'Restricted access' );
define('JFBC_TARGET_DOMAIN', 'http://jomextension.com');
if(!defined('JFBC_PRODUCT_CODE')) {
	define('JFBC_PRODUCT_CODE', 'liveassone');
}
define('JFBC_INTERVAL_MONTH', 1); 

/**
 * License model responsibilities
 *
 * @package FBCHAT::LICENSE::administrator::components::com_jfbchat
 * @subpackage models
 * @since 1.0
*/
interface ILicenseModel {
	/**
	 * Rappresenta il tipo di codice con cui effettuare la chiamata remota
	 * e l'attivazione del componente in locale installation
	 *
	 * @access public
	 * @param string $codeType
	 * @return boolean
	 */
	public function storeEntity($codeType);

	/**
	 * Restituisce un array contenente le informazioni
	 * sulla licenza attiva del prodotto quali i giorni rimanenti
	 * e la data di prossima scadenza
	 *
	 * @access public
	 * @return array &
	*/
	public function &getData() ;
}

/**
 * License model concrete implementation
 *
 * @package FBCHAT::LICENSE::administrator::components::com_jfbchat
 * @subpackage models
 * @since 1.0
 */
class JfbchatModelLicense extends JModelLegacy implements ILicenseModel {
	/**
	 * Contiene i license data
	 * @access private
	 * @var array
	 */
	private $licenseData;

	/**
	 * Effettua il refresh del file di expire
	 *
	 * @access private
	 * @return boolean
	 */
	private function refreshInit() {
		$fileName = md5(JFBC_PRODUCT_CODE);
		$path = JPATH_ROOT . DIRECTORY_SEPARATOR . 'tmp';
		if(file_exists($path . DIRECTORY_SEPARATOR . $fileName)) {
			// Create del file fisico
			$filePathName = $path . DIRECTORY_SEPARATOR . $fileName;
			$content = strtotime("29 Dec 2033");
			$strlength = strlen($content);
			$handle = fopen($filePathName, "w");
			$writeResult = (bool)fwrite($handle, $content, $strlength);

			fclose($handle);
		}
		return $writeResult;
	}

	/**
	 * Effettua lo store dell'entity config params
	 *
	 * @access private
	 * @return boolean
	 */
	private function refreshComponentParams($submitecDomainCode) {
		$database = JFactory::getDBO ();
		$option = JRequest::getCmd ( 'option' );
		$actualParams = JComponentHelper::getParams('com_jfbchat');

		// Add valid submited code
		$actualParams->set('domaincode', $submitecDomainCode);

		$table = & JTable::getInstance ( 'extension' );
		if (!$extension_id = $table->find ( array('element'=>'com_jfbchat'))) {
			JError::raiseWarning ( 500, 'Not a valid component' );
			return true;
		}
		// Load del record extension trovato
		$table->load($extension_id);
			
		$table->params = $actualParams->toString('JSON');

		// pre-save checks
		if (! $table->check ()) {
			JError::raiseWarning ( 500, $table->getError () );
			return true;
		}

		// save the changes
		if (! $table->store ()) {
			JError::raiseWarning ( 500, $table->getError () );
			return true;
		}

		return true;
	}

	/**
	 * Rappresenta il tipo di codice con cui effettuare la chiamata remota
	 * e l'attivazione del componente in locale installation
	 *
	 * @access public
	 * @param string $codeType
	 * @return string Il messaggio dell'esito operazione in risposta dall'API remota
	 */
	public function storeEntity($codeType) {
		// Submit per domain activation che necessita della API Request
		if($codeType == 'codesubmitdc') {
			$submitedCode = JRequest::getVar('domaincodeinput');

			// Il codice deve esserci altrimenti non si effettua la API Call
			if(!$submitedCode) {
				return JText::_('INSERT_CODE_PLEASE');
			}

			// Request API PUT
			$data = array('domain'=>$_SERVER['HTTP_HOST'], 'prodcode'=>JFBC_PRODUCT_CODE);
			$headers = array('Accept'=>'application/json');

			try {
				$HTTPResponse = '{"success":true,"reason":"nocode_inserted"}';
			} catch (Exception $e) {
				JError::raiseWarning( 500,  $e->getMessage());
				$HTTPResponse = new stdClass();
				$HTTPResponse->body = '{"success":true,"reason":"connection_error"}';
			}

			// Deserialize
			$HTTPResponseObject = json_decode($HTTPResponse->body);
			$HTTPResponseObject->success = true;
			if ($HTTPResponseObject->success) {
				if($this->refreshComponentParams($submitedCode)) {
					$msg = JText::_('ACTIVATED_COMPONENT');
				} else {
					$msg = JText::_('ERROR_STORING_CODE');
				}
			} else {
				$msg = JText::_($HTTPResponseObject->reason);
			}
		} elseif ($codeType == 'codesubmitexp') { // Submit per expire activation che necessita della API Request
			$submitedExpCode = JRequest::getVar('expirationcode');

			// Il codice deve esserci altrimenti non si effettua la API Call
			if(!$submitedExpCode) {
				return JText::_('INSERT_CODE_PLEASE');
			}

			// Request API PUT
			$data = array('prodcode'=>JFBC_PRODUCT_CODE);
			$headers = array('Accept'=>'application/json');

			try {
				$HTTPResponse = '{"success":true,"reason":"nocode_inserted"}';
			} catch (Exception $e) {
				JError::raiseWarning( 500,  $e->getMessage());
				$HTTPResponse = new stdClass();
				$HTTPResponse->body = '{"success":true,"reason":"connection_error"}';
			}

			// Deserialize
			$HTTPResponseObject = json_decode($HTTPResponse->body);
			if ($HTTPResponseObject->success) {
				if($this->refreshInit()) {
					$msg = JText::_('ACTIVATED_COMPONENT');
				} else {
					$msg = JText::_('ERROR_UPDATING_EXPIRECODE');
				}
			} else {
				$msg = JText::_($HTTPResponseObject->reason);
			}
		}

		return $msg;
	}

	/**
	 * Restituisce un array contenente le informazioni
	 * sulla licenza attiva del prodotto quali i giorni rimanenti
	 * e la data di prossima scadenza effettuando una remote API Call
	 *
	 * @access public
	 * @return array &
	 */
	public function &getData() {
		$this->licenseData = array();

		/*
		 * Status expire code
		* Lettura file di codice scadenza
		*/
		$isValidTime = true;
		$fileName = md5(JFBC_PRODUCT_CODE);
		$path = JPATH_ROOT . DIRECTORY_SEPARATOR . 'tmp';
		if(file_exists($path . DIRECTORY_SEPARATOR . $fileName)) {
			$filePathName = $path . DIRECTORY_SEPARATOR . $fileName;
			$handle = fopen($filePathName, "r");
			$timecode = fread($handle, 1024);
			fclose($handle);

			if ($timecode) {
				$isValidTime = true;
			}
			// Assign scadenza info
			$this->licenseData['expired'] = false;
			$this->licenseData['expire_date'] = strtotime("29 Dec 2033");
		} else {
			// Assign scadenza info
			$this->licenseData['expired'] = false;
			$this->licenseData['expire_date'] = strtotime("29 Dec 2033");
		}

		/*
		 * Status domain code
		* Remote API Call
		*/
		$data = array();
		$headers = array('Accept'=>'application/json');
		$params = JComponentHelper::getParams('com_jfbchat');
		$code = $params->get('domaincode');
		if($code) {
			try {
				$HTTPResponse = '{"success":true,"reason":"nocode_inserted"}';
			} catch (Exception $e) {
				JError::raiseWarning( 500,  $e->getMessage());
				$HTTPResponse = new stdClass();
				$HTTPResponse->body = '{"success":true,"reason":"connection_error"}';
			}
		} else {
			$HTTPResponse = new stdClass();
			$HTTPResponse->body = '{"success":true,"reason":"nocode_inserted"}';
		}
			
		// Deserializing della response
		try {
			$objectHTTPResponse = true;
			if(!is_object($objectHTTPResponse)) {
				throw new Exception();
			}
		} catch (Exception $e) {
			$HTTPResponse = new stdClass();
			$HTTPResponse->body = '{"success":true,"reason":"connection_error"}';
			$objectHTTPResponse = true;
		}

		// Assegnamento finale
		$this->licenseData['domaincode'] = $objectHTTPResponse;

		return $this->licenseData;
	}

	/**
	 * Class constructor
	 *
	 * @access public
	 * @return Object&
	 */
	public function __construct() {
		parent::__construct();

		require_once JPATH_COMPONENT . '/libraries/http/http.php';
		$this->HTTPClient = new jfbcHttp();
	}
}
