<?php
// namespace administrator\components\com_jfbchat\models;
/** 
 * @package FBCHAT::REALSTATS::administrator::components::com_jfbchat
 * @subpackage models
 * @author 2Punti - Marco Biagioni
 * @copyright (C) 2012 - 2PUNTI SRL
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */
defined ( '_JEXEC' ) or die ( 'Restricted access' ); 

/**
 * Model responsibilities contract
 *
 * @package FBCHAT::REALSTATS::administrator::components::com_jfbchat
 * @subpackage models
 * @since 1.0
 */
interface IRealstatsModel {
	/**
	 * Config accessor method
	 *
	 * @access public
	 * @return Object&
	 */
	public function &getConfig();
	
	/**
	 * Main get data method
	 *
	 * @access public
	 * @return Object[]
	 */
	public function getData();
}

/**
 * Realtime stats concrete implementation
 * Incorpora in un single operational responsibility tutto il calcolo dei dati
 * da fornire in JSON format alla JS APP di frontend per il visual rendering
 *
 * @package FBCHAT::REALSTATS::administrator::components::com_jfbchat
 * @subpackage models
 * @since 1.0
 */
class JfbchatModelRealstats extends JModelLegacy implements IRealstatsModel {
	/**
	 * Tempo oltre il quale considerare un utente in #__fbchat_realstats
	 * come non pi presente sul sito e quindi da non inserire pi
	 * nelle statistiche
	 *
	 * @access private
	 * @var int
	 */
	private $maxInactivityTime;
	
	/**
	 * Query chunk se gruppo Public selezionato per i valid customers
	 *
	 * @access private
	 * @var array
	 */
	private $queryChunk;
	
	/**
	 * Data container da renderizzare in JSON ad opera della view
	 *
	 * @access private
	 * @var array
	 */
	private $data;
	
	/**
	 * Component configuration pointer
	 *
	 * @access private
	 * @var Object&
	 */
	private $config;
	
	/**
	 * Setted agents users id
	 */
	private $settedAgents;
	
	/**
	 * Customers groups per considerarli validi
	 */
	private $customerGroups;
	
	/**
	 * Numero di utenti totali attualmente presenti sul sito
	 * Si basa su una COUNT sui records validi di #__fbchat_realstats 
	 *
	 * @access private
	 * @return int 
	 */
	private function totalUserCount() {
		// Build query
		$query = "SELECT COUNT(*) FROM #__fbchat_realstats" .
				 "\n WHERE " . $this->_db->quoteName('lastupdate_time') . " > " . (int)(time()-$this->maxInactivityTime);
		$this->_db->setQuery($query);
		$countedResults = $this->_db->loadResult();
		
		$totalUsers = new stdClass();
		$totalUsers->source = JText::_('TotalUsers');
		$totalUsers->value = $countedResults;
		
		return $totalUsers;
	}
	
	/**
	 * Numero di utenti totale attualmente presenti sul sito per pagina
	 * Si basa su una COUNT sui records validi di #__fbchat_realstats con GROUP BY nowpage
	 *
	 * @access private
	 * @return array
	 */
	private function totalUserCountByPage() {
		// Build query
		$query = "SELECT COUNT(session_id_person) AS numusers,  DATE_FORMAT(FROM_UNIXTIME(MAX(lastupdate_time)), '%H:%i:%s, %d/%m/%Y') AS lastvisit, nowpage FROM #__fbchat_realstats" .
				 "\n WHERE " . $this->_db->quoteName('lastupdate_time') . " > " . (int)(time()-$this->maxInactivityTime) .
				 "\n GROUP BY " . $this->_db->quoteName('nowpage');
		$this->_db->setQuery($query);
		$usersPerPage = $this->_db->loadObjectList(); 
		 
		return $usersPerPage;
	}
	
	/**
	 * Numero di utenti totali attualmente presenti sul sito e pagina in cui si trovano
	 * Si basa su una SELECT incondizionata sui records validi di #__fbchat_realstats 
	 *
	 * @access private
	 * @return array
	 */
	private function usersPageOn() {
		// Build query
		$query = "SELECT stats.nowpage, stats.current_name, DATE_FORMAT(FROM_UNIXTIME(stats.lastupdate_time), '%H:%i:%s, %d/%m/%Y') AS lastupdatetime, sess.username, userg.title AS usertype, users.name" .
				 "\n FROM #__fbchat_realstats AS stats" .
				 "\n INNER JOIN #__session AS sess" .
				 "\n ON sess.session_id = stats.session_id_person" . 
				 "\n LEFT JOIN #__users AS users" .
				 "\n ON sess.userid = users.id" .
				 "\n LEFT JOIN #__user_usergroup_map AS map" .
				 "\n ON map.user_id = users.id" .
				 "\n LEFT JOIN #__usergroups AS userg" .
				 "\n ON map.group_id = userg.id" .
				 "\n WHERE " . $this->_db->quoteName('lastupdate_time') . " > " . (int)(time()-$this->maxInactivityTime) .
				 "\n AND sess.client_id = 0" . 
				 "\n AND (map.group_id IN (" . implode(',', $this->customerGroups) . ")" . $this->queryChunk .
				 "\n OR sess.userid IN (" . implode(',', $this->settedAgents) . "))";
		$this->_db->setQuery($query);
		$users = $this->_db->loadObjectList(); 
		
		return $users;
	}
	
	/**
	 * Numero di utenti customers visitatori
	 * Si basa su una COUNT sui records validi di #__fbchat_realstats 
	 * in JOIN con #__session di cui si valuta il guest = 1 e la doppia
	 * condizione di id utente non presente tra quelli degli agents designati
	 * e group id compreso tra i gruppi customers designed
	 *
	 * @access private
	 * @return int 
	 */
	private function totalVisitorsCustomers() {
		// Build query
		$query = "SELECT COUNT(*) FROM #__fbchat_realstats AS stats" .
				 "\n INNER JOIN #__session AS sess" .
				 "\n ON sess.session_id = stats.session_id_person" .
				 "\n LEFT JOIN #__user_usergroup_map AS map" .
				 "\n ON map.user_id = sess.userid" .
				 "\n WHERE " . $this->_db->quoteName('lastupdate_time') . " > " . (int)(time()-$this->maxInactivityTime) .
				 "\n AND sess.client_id = 0" .
				 "\n AND sess.guest = 1" .
				 "\n AND (map.group_id IN (" . implode(',', $this->customerGroups) . ")" . $this->queryChunk . ")" .
				 "\n AND sess.userid NOT IN (" . implode(',', $this->settedAgents) . ")";
		$this->_db->setQuery($query);
		$countedResults = $this->_db->loadResult();
		
		$customersVisitor = new stdClass();
		$customersVisitor->source = JText::_('CustomersVisitor');
		$customersVisitor->value = (int)$countedResults;
		
		return $customersVisitor;
	}
	
	/**
	 * Numero di utenti customers loggati
	 * Si basa su una COUNT sui records validi di #__fbchat_realstats 
	 * in JOIN con #__session di cui si valuta il guest = 0 e la doppia
	 * condizione di id utente non presente tra quelli degli agents designati
	 * e group id compreso tra i gruppi customers designed
	 *
	 * @access private
	 * @return int 
	 */
	private function totalLoggedCustomers() {
		// Build query
		$query = "SELECT COUNT(*) FROM #__fbchat_realstats AS stats" .
				 "\n INNER JOIN #__session AS sess" .
				 "\n ON sess.session_id = stats.session_id_person" .
				 "\n LEFT JOIN #__user_usergroup_map AS map" .
				 "\n ON map.user_id = sess.userid" .
				 "\n WHERE " . $this->_db->quoteName('lastupdate_time') . " > " . (int)(time()-$this->maxInactivityTime) .
				 "\n AND sess.client_id = 0" .
				 "\n AND sess.guest = 0" .
				 "\n AND (map.group_id IN (" . implode(',', $this->customerGroups) . ")" . $this->queryChunk . ")" .
				 "\n AND sess.userid NOT IN (" . implode(',', $this->settedAgents) . ")";
		$this->_db->setQuery($query);
		$countedResults = $this->_db->loadResult();
		
		$customersLogged = new stdClass();
		$customersLogged->source = JText::_('CustomersLogged');
		$customersLogged->value = (int)$countedResults;
		
		return $customersLogged;
	}
	
	/**
	 * Numero di agenti totali loggati
	 * Si basa su una COUNT dei record di #__fbchat_realstats in JOIN
	 * con #__session per i records che hanno guest=0, userid compreso
	 * tra gli agents abilitati e ovviamente lastupdate_time valido
	 *
	 * @access private
	 * @return int 
	 */
	private function totalAgents() {
		// Build query
		$query = "SELECT COUNT(*) FROM #__fbchat_realstats AS stats" .
				 "\n INNER JOIN #__session AS sess" .
				 "\n ON sess.session_id = stats.session_id_person" .
				 "\n WHERE " . $this->_db->quoteName('lastupdate_time') . " > " . (int)(time()-$this->maxInactivityTime) .
				 "\n AND sess.client_id = 0" .
				 "\n AND sess.guest = 0" . 
				 "\n AND sess.userid IN (" . implode(',', $this->settedAgents) . ")";
		$this->_db->setQuery($query);
		$countedResults = $this->_db->loadResult();
		
		$agentsLogged = new stdClass();
		$agentsLogged->source = JText::_('TotalAgents');
		$agentsLogged->value = (int)$countedResults;
		
		return $agentsLogged;
	}
	
	/** 
	 * Numero di utenti customers loggati
	 * Si basa su una COUNT sui records validi di #__fbchat_realstats 
	 * in JOIN con #__fbchat_session_relations sul campo p.key session_id_customer
	 *
	 * @access private
	 * @return int  
	 */
	private function totalServedCustomers() {
		// Build query
		$query = "SELECT COUNT(*) FROM #__fbchat_session_relations AS rel" .
				 "\n INNER JOIN #__fbchat_realstats AS stats" .
				 "\n ON rel.session_id_customer = stats.session_id_person" .
				 "\n INNER JOIN #__fbchat_realstats AS stats_agents" .
				 "\n ON rel.session_id_agent = stats_agents.session_id_person" .
				 "\n WHERE stats.lastupdate_time > " . (int)(time()-$this->maxInactivityTime) .
				 "\n AND stats_agents.lastupdate_time > " . (int)(time()-$this->maxInactivityTime);
		$this->_db->setQuery($query);
		$countedResults = $this->_db->loadResult();
		
		$servedCustomers = new stdClass();
		$servedCustomers->source = JText::_('ServedCustomers');
		$servedCustomers->value = (int)$countedResults;
		
		return $servedCustomers;
	}
	 
	/**
	 * Config accessor method
	 *
	 * @access public 
	 * @return Object&
	 */
	public function &getConfig() {
		return $this->config;
	}
	  
	/**
	 * Main get data method Single Responsibility operation
	 *
	 * @access public
	 * @param boolean $pieRequest
	 * @param boolean $initRequest
	 * @return Object[]
	 */
	public function getData($pieRequest = false, $initRequest = false) { 
		// Esclusione pieRequest data per total
		if(!$pieRequest) { 
			if(!$initRequest) {
				$this->data[] = $this->usersPageOn(); 
				$this->data[] = $this->totalUserCountByPage();
			}
			$this->data[] = $this->totalUserCount();
			$this->data[] = $this->totalServedCustomers();
		}
  
		// START Totale pie
		$this->data[] = $this->totalVisitorsCustomers();
	  	$this->data[] = $this->totalLoggedCustomers();
	  	$this->data[] = $this->totalAgents();  
		// END Totale pie
		   
		return $this->data;
	}
	
	/**
	 * Class contructor
	 *  
	 * @access public
	 * @return Object&
	 */
	public function __construct() {
		parent::__construct();
		
		$this->data = array();
		$this->config = &JComponentHelper::getParams('com_jfbchat'); 
		$this->maxInactivityTime = $this->config->get('maxlifetime_session');
		
		$this->queryChunk = null;
		$this->customerGroups = $this->config->get('customer_gids');
		// Query chunk settings
		if(is_array($this->customerGroups)) {
			if(in_array(1, $this->customerGroups)) {
				$this->queryChunk = " OR ISNULL(map.group_id)";
			}
		}
		// Setting degli utenti agents designati
		$query = "SELECT userid FROM #__fbchat_agents";
		$this->_db->setQuery($query);
		$this->settedAgents = $this->_db->loadColumn();
		if(!count($this->settedAgents)) {
			$this->settedAgents = array(-1);
		}
	}
} 