<?php
// namespace administrator\components\com_jfbchat\models;
/** 
 * @package FBCHAT::SUGGESTIONS::administrator::components::com_jfbchat
 * @subpackage models
 * @author 2Punti - Marco Biagioni
 * @copyright (C) 2012 - 2PUNTI SRL
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */
defined ( '_JEXEC' ) or die ( 'Restricted access' ); 

/**
 * Suggestions model responsibilities contract
 *
 * @package FBCHAT::SUGGESTIONS::administrator::components::com_jfbchat
 * @subpackage models
 * @since 1.0
 */
interface ISuggestionsModel { 
	/**
	 * Main get data method
	 * @access public
	 * @return Object[]
	 */
	public function getData();

	/**
	 * Counter result set
	 * @access public
	 * @return int
	 */
	public function getTotal();

	/**
	 * Effettua il load dell'entity singola da orm table
	 * @access public
	 * @param int $id
	 * @return Object&
	 */
	public function loadEntity($id);

	/**
	 * Storing record tramite $table
	 * @access public
	 * @return mixed
	 */
	public function storeEntity();

	/**
	 * Cancel editing entity
	 * 
	 * @param int $id
	 * @access public
	 * @return boolean
	 */
	public function cancelEntity($id);

	/**
	 * Imposta la suggestion come il messaggio predefinito da mostrarsi nel fumetto
	 * di frontend autosuggestion
	 *
	 * @access public
	 * @return void
	 */
	public function setAutoInviteFlag($id, $state);
	/**
	 * Modifica sul DB lo stato di pubblicazione
	 * @param int $id
	 * @param string $state
	 * @access public
	 * @return boolean
	 */
	public function publishEntity($id, $state);
}

/**
 * Suggestionsmodel concrete implementation
 *
 * @package FBCHAT::SUGGESTIONS::administrator::components::com_jfbchat
 * @subpackage models
 * @since 1.0
 */
class JfbchatModelSuggestions extends JModelLegacy implements ISuggestionsModel {
	/**
	 * @AttributeType Object[]
	 */
	private $records;

	/**
	 * @access private
	 */
	private function buildListQuery() {
		// WHERE
		$where = array();
		$whereString = null;
		    
		if (count($where)) {
			$whereString = "\n WHERE " . implode ("\n AND ", $where);
		}
		   
		$query = "SELECT *"
				. "\n FROM #__fbchat_suggestions AS a"
				. $whereString;
		return $query;
	}

	/**
	 * @access public
	 */
	public function getData() {
		// Build query 
		$query = $this->buildListQuery();
		$this->_db->setQuery( $query, $this->getState('limitstart'), $this->getState('limit') );
		$result = $this->_db->loadObjectList();

		return $result;
	}

	/**
	 * @access public
	 */
	public function getTotal() {
		// Build query 
		$query = $this->buildListQuery();
		$this->_db->setQuery( $query);
		$result = count( $this->_db->loadColumn() );

		return $result;
	}

	/**
	 * Effettua il load dell'entity singola da orm table
	 * @access public
	 * @param int $id
	 * @return Object&
	 */
	public function loadEntity($id) {
		// load table record
		$table = $this->getTable('Suggestions');
		$table->load($id);
		 
		if(!$table){
			JError::raiseNotice ( 400, $this->_db->getErrorMsg () );
			return false;
		}
		return $table;
	}

	/**
	 * Storing record tramite $table
	 * @access public
	 * @return mixed
	 */
	public function storeEntity() {
		$table = $this->getTable('Suggestions');
		try {
			if (! $table->bind ($_POST)) {
				throw new Exception($table->getError ());
			}
		
			if (! $table->check ( )) {
				throw new Exception($table->getError ());
			}
				
			if (! $table->store (false)) {
				throw new Exception($table->getError ());
			}
		} catch(Exception $e) {
			switch ($e->getCode()) {
				default:
					JError::raiseNotice(100, $e->getMessage());
			}
			return false;
		}
			
		return $table;
	}
	
	/**
	 * Cancel editing entity
	 * 
	 * @param int $id
	 * @access public
	 * @return boolean
	 */
	public function cancelEntity($id) {
		$table = $this->getTable('Suggestions');
		try { 
			if (! $table->load($id)) {
				throw new Exception($table->getError ());
			}
			if (! $table->checkin()) {
				throw new Exception($table->getError ());
			} 
		} catch(Exception $e) {
			switch ($e->getCode()) {
				default:
					JError::raiseNotice(100, $e->getMessage());
			}
			return false;
		}
			
		return true;
	}
	
	/**
	 * Delete entity
	 *
	 * @param array $ids
	 * @access public
	 * @return boolean
	 */
	public function deleteEntity($ids) {
		$table = $this->getTable('Suggestions');
		
		// Ciclo su ogni entity da cancellare
		foreach ($ids as $id) {
			try {
				if (! $table->delete($id)) {
					throw new Exception($table->getError ());
				} 
			} catch(Exception $e) {
				switch ($e->getCode()) {
					default:
						JError::raiseNotice(100, $e->getMessage());
				}
				return false;
			} 
		}
		 
		return true;
	}
 
	/**
	 * Imposta su DB come suggerimento di default
	 * @param int $id
	 * @param string $state
	 * @access public
	 * @return boolean
	 */
	public function setAutoInviteFlag($id, $state) {
		// Reset all messages state
		$query = "UPDATE #__fbchat_suggestions" .
				 "\n SET " . $this->_db->quoteName('autoinvite') . " = NULL";
		$this->_db->setQuery($query);
		$this->_db->execute();
		
		// Table load
		$table = $this->getTable('Suggestions');
		$table->load($id);
		switch($state){
			case 'nodefaultinvite':
				$table->autoinvite = null;
				break;
				
			case 'defaultinvite':
				$table->autoinvite = 1;
				break; 
		}
		 
		if(!$table->store(true)){
			JError::raiseNotice ( 400, $this->_db->getErrorMsg () );
			return false;
		}
		return true;
	}
	
	/**
	 * Modifica sul DB lo stato di pubblicazione
	 * @param int $id
	 * @param string $state
	 * @access public
	 * @return boolean
	 */
	public function publishEntity($id, $state) {
		// Table load
		$table = $this->getTable('Suggestions');
		$table->load($id);
		switch($state){
			case 'unpublish':
				$table->published = null;
				break;
				
			case 'publish':
				$table->published = 1;
				break; 
		}
		 
		if(!$table->store(true)){
			JError::raiseNotice ( 400, $this->_db->getErrorMsg () );
			return false;
		}
		return true;
	}
}
?>