<?php
// namespace administrator\components\com_jfbchat\views\messages;
/** 
 * @package FBCHAT::MESSAGES::administrator::components::com_jfbchat
 * @subpackage views
 * @subpackage messages
 * @author 2Punti - Marco Biagioni
 * @copyright (C) 2012 - 2PUNTI SRL
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */
defined ( '_JEXEC' ) or die ( 'Restricted access' );  
define ( 'INDEX_WORKED', 'worked'); 
jimport('joomla.html.pagination');

/**
 * User leaved messages view contract
 *
 * @package FBCHAT::MESSAGES::administrator::components::com_jfbchat
 * @subpackage views
 * @subpackage messages
 * @since 1.0
 */
interface ILAMessagesView { 
	/**
	 * Mostra la visualizzazione dettaglio del record singolo
	 * @param Object& $row
	 * @access public
	 */
	public function showEntity(&$row);
	
	/**
	 * Effettua l'output view del file in attachment al browser
	 *
	 * @access public
	 * @param string $contents
	 * @param int $size
	 * @param array& $fieldsFunctionTransformation
	 * @return void
	 */
	public function sendCSVMessages($data, &$fieldsFunctionTransformation);
}

/**
 * User leaved messages view implementation
 *
 * @package FBCHAT::MESSAGES::administrator::components::com_jfbchat
 * @subpackage views
 * @since 1.0
 */
class JfbchatViewLAMessages extends JViewLegacy implements ILAMessagesView {
	/**
	 * Add the page title and toolbar.
	 *
	 * @since	1.6
	 */
	protected function addShowEntityToolbar() {
		JToolBarHelper::title( JText::_( 'MESSAGE_DETAILS' ), 'jfbchat' );
		JToolBarHelper::apply( 'lamessages.applyEntity', 'apply');
		JToolBarHelper::save( 'lamessages.saveEntity', 'save');
		JToolBarHelper::custom( 'lamessages.responsemessage', 'responsemessage', 'responsemessage', 'RESPONSE_LEAVED_MESSAGES', false);
		JToolBarHelper::custom('lamessages.display', 'back', 'back', 'LIST_LEAVED_MESSAGES', false);
	}
	
	/**
	 * Add the page title and toolbar.
	 *
	 * @since	1.6
	 */
	protected function addDisplayToolbar() {
		JToolBarHelper::title( JText::_( 'LIST_LEAVED_MESSAGES' ), 'jfbchat' );
		JToolBarHelper::editList('lamessages.editentity', 'VIEW_MESSAGE_DETAILS');
		JToolBarHelper::deleteList(JText::_('DELETE_LAMESSAGES'), 'lamessages.deleteentity');
		JToolBarHelper::custom('lamessages.exportmessages', 'export', 'export', 'EXPORT_MSG', false);
		JToolBarHelper::custom('cpanel.display', 'config', 'config', 'CPANEL', false);
	}
	
	/**
	 * Inietta le costanti lingua nel JS Domain con il solito name mapping
	 * @access protected
	 * @param $translations Object&
	 * @param $document Object&
	 * @return void
	 */
	protected function injectJsTranslations(&$translations, &$document) {
		$jsInject = null;
 		// Do translations
		foreach ( $translations as $translation ) {
			$jsTranslation = strtoupper ( $translation );
			$translated = JText::_( $jsTranslation );
			$jsInject .= <<<JS
				jfbc$translation = '{$translated}'; 
JS;
		}
		$document->addScriptDeclaration($jsInject);
	}
	/**
	 * Default listEntities
	 * @access public
	 */
	public function display() {
		$doc = JFactory::getDocument ();
		$doc->addStylesheet ( JURI::root ( true ) . '/administrator/components/com_jfbchat/css/generic.css' );
		JHTML::_('behavior.calendar');
		// Get main records
		$rows = $this->get('Data');
		$lists = $this->get('Lists');
		$total = $this->get('Total');
		
		$orders = array();
		$orders['order'] = $this->getModel()->getState('order');
		$orders['order_Dir'] = $this->getModel()->getState('order_dir');
		// Pagination view object model state populated
		$pagination = new JPagination( $total, $this->getModel()->getState('limitstart'), $this->getModel()->getState('limit') );
		$dates = array('start'=>$this->getModel()->getState('fromPeriod'), 'to'=>$this->getModel()->getState('toPeriod'));
		 
		$this->assignRef('pagination',	$pagination);
		$this->assignRef('order', $this->getModel()->getState('order'));
		$this->assignRef('searchword', $this->getModel()->getState('searchword'));
		$this->assignRef('lists', $lists);
		$this->assignRef('orders', $orders);
		$this->assignRef('items', $rows);
		$this->assignRef('option', $this->getModel()->getState('option'));
		$this->assignRef('dates', $dates); 
		
		// Aggiunta toolbar
		$this->addDisplayToolbar();
		
		// Output del template
		parent::display();
	}

	/**
	 * Mostra la visualizzazione dettaglio del record singolo
	 * @param Object& $row
	 * @access public
	 */
	public function showEntity(&$row) {
		// Aggiunta toolbar
		$this->addShowEntityToolbar();
		
		$doc = JFactory::getDocument ();
		$doc->addStylesheet ( JURI::root ( true ) . '/administrator/components/com_jfbchat/css/generic.css' );
		$doc->addScript(JURI::root(true) . '/components/com_jfbchat/js/jquery.js');
		$doc->addScript(JURI::root(true) . '/components/com_jfbchat/js/jquery.noconflict.js');
		$editor = &JFactory::getEditor();
		
		// Inject js translations
		$translations = array( 'validation_error', 'validation_error_subject' );
		$this->injectJsTranslations($translations, $doc);
		
		$this->addJSValidateScript($editor);
		
		$this->assignRef('option', $this->getModel()->getState('option'));
		$this->assignRef('record', $row);
		$this->assignRef('editor', $editor);
		
		$this->setLayout('messages');
		parent::display('details');
	}
	
	/**
	 * Effettua l'output view del file in attachment al browser
	 *
	 * @access public
	 * @param string $contents
	 * @param int $size
	 * @param array& $fieldsFunctionTransformation
	 * @return void
	 */
	public function sendCSVMessages($data, &$fieldsFunctionTransformation) {
		$delimiter = ';';
		$enclosure = '"';
		// Clean dirty buffer
		ob_end_clean();
		// Open buffer
		ob_start();
		// Open out stream
		$outstream = fopen("php://output", "w");
		// Funzione di scrittura nell'output stream
		function __outputCSV(&$vals, $key, $userData) {
			// Fields value transformations 
			if(isset($vals[INDEX_WORKED]) && (int)$vals[INDEX_WORKED]) {
				$vals[INDEX_WORKED] = $vals[INDEX_WORKED] == 1 ? JText::_('YESWORKED') : JText::_('NOWORKED');
			}
			fputcsv($userData[0], $vals, $userData[1], $userData[2]); // add parameters if you want
		}
		// Echo delle intestazioni
		__outputCSV($fieldsFunctionTransformation, null, array($outstream, $delimiter, $enclosure));
		// Output di tutti i records
		array_walk($data, "__outputCSV", array($outstream, $delimiter, $enclosure));
		fclose($outstream);
		// Recupero output buffer content
		$contents = ob_get_clean();
		$size = strlen($contents);
	
	
		header ( 'Pragma: public' );
		header ( 'Cache-Control: must-revalidate, post-check=0, pre-check=0' );
		header ( 'Expires: ' . gmdate ( 'D, d M Y H:i:s' ) . ' GMT' );
		header ( 'Content-Disposition: attachment; filename="users.csv"' );
		header ( 'Content-Type: text/plain' );
		header ( "Content-Length: " . $size );
		echo $contents;
			
		exit ();
	}
	
	/**
	 * Gestisce la validazione innescata dalla submitbutton
	 * @param Object& $editor
	 */
	private function addJSValidateScript($editor) {
	ob_start();
	     
	$str = <<<EOD
	<script type="text/javascript" >
	//<![CDATA[
		window.onload = function() { 
			var batteryCtrls = new Array('email_subject');
			jQuery.each(batteryCtrls, function(k, value) { 
				var control= jQuery('input[name=' + value + '], select[name=' + value + ']');
			  	control.bind('blur', function(event){rivalida(event);});
			});
 
			function rivalida(event) { 
			    if(event.target.style.borderColor && event.target.value != '') {
			    	event.target.style.borderColor = '#CCCCCC';
			    }
			}
		}
		Joomla.submitbutton = function(pressbutton) {
			var form = document.adminForm;
			var msg = '' ;
	  		// do field validation
			var text = {$editor->getContent( 'response' )};
				  
			if (pressbutton != 'lamessages.responsemessage') { 
				submitform(pressbutton);
				return true;
			}
				     
			// do field validation
			if (form.email_subject.value == "") { 
		   		msg += jfbcvalidation_error_subject;
		 		form.email_subject.style.borderColor = "#FF0000";
		 	}
			
		    if (text == "") { 
		   		msg += '\\n' + jfbcvalidation_error;
		 		form.response.style.borderColor = "#FF0000";
		 	}  
			 
			if(msg) {
				alert(msg);
			} else { 
				{$editor->save( 'response' )}
				submitform(pressbutton); 
			}
		}
	//]]>
	</script>
EOD;
	
	echo $str;
	ob_end_flush();
	
		}
}