<?php
// namespace administrator\components\com_jfbchat\views\messages;
/** 
 * @package FBCHAT::MESSAGES::administrator::components::com_jfbchat
 * @subpackage views
 * @subpackage messages
 * @author 2Punti - Marco Biagioni
 * @copyright (C) 2012 - 2PUNTI SRL
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */
defined ( '_JEXEC' ) or die ( 'Restricted access' );
define ( 'INDEX_SENT', 'sent');
define ( 'INDEX_READ', 'read');
define ( 'INDEX_MESSAGE', 'message'); 
jimport('joomla.html.pagination');

/**
 * User messages view contract
 *
 * @package FBCHAT::MESSAGES::administrator::components::com_jfbchat
 * @subpackage views
 * @since 1.0
 */
interface IMessagesView { 
	/**
	 * Mostra la visualizzazione dettaglio del record singolo
	 * @param Object& $row
	 * @access public
	 */
	public function showEntity(&$row);
	
	/**
	 * Effettua l'output view del file in attachment al browser
	 *
	 * @access public
	 * @param string $contents
	 * @param int $size
	 * @param array& $fieldsFunctionTransformation
	 * @return void
	 */
	public function sendCSVMessages($data, &$fieldsFunctionTransformation);
}

/**
 * User messages view implementation
 *
 * @package FBCHAT::MESSAGES::administrator::components::com_jfbchat
 * @subpackage views
 * @since 1.0
 */
class JfbchatViewMessages extends JViewLegacy implements IMessagesView { 
	/**
	 * Add the page title and toolbar.
	 *
	 * @since	1.6
	 */
	protected function addShowEntityToolbar() {
		JToolBarHelper::title( JText::_( 'MESSAGE_DETAILS' ), 'jfbchat' );  
		JToolBarHelper::custom('messages.display', 'back', 'back', 'LIST_MESSAGES', false);
	}
	
	/**
	 * Add the page title and toolbar.
	 *
	 * @since	1.6
	 */
	protected function addDisplayToolbar() {
		JToolBarHelper::title( JText::_( 'LIST_MESSAGES' ), 'jfbchat' );
		JToolBarHelper::editList('messages.showentity', 'VIEW_MESSAGE_DETAILS');
		JToolBarHelper::deleteList(JText::_('DELETE_MESSAGES'), 'messages.deleteentity');
		JToolBarHelper::custom('messages.exportmessages', 'export', 'export', 'EXPORT_MSG', false);
		JToolBarHelper::custom('cpanel.display', 'config', 'config', 'CPANEL', false);
	}
	
	/**
	 * Default listEntities
	 * @access public
	 */
	public function display() {
		$doc = JFactory::getDocument ();
		$doc->addStylesheet ( JURI::root ( true ) . '/administrator/components/com_jfbchat/css/generic.css' );
		JHTML::_('behavior.calendar');
		// Get main records
		$rows = $this->get('Data');
		$lists = $this->get('Lists');
		$total = $this->get('Total');
		
		$orders = array();
		$orders['order'] = $this->getModel()->getState('order');
		$orders['order_Dir'] = $this->getModel()->getState('order_dir');
		// Pagination view object model state populated
		$pagination = new JPagination( $total, $this->getModel()->getState('limitstart'), $this->getModel()->getState('limit') );
		$dates = array('start'=>$this->getModel()->getState('fromPeriod'), 'to'=>$this->getModel()->getState('toPeriod'));
		 
		$this->assignRef('pagination',	$pagination);
		$this->assignRef('order', $this->getModel()->getState('order'));
		$this->assignRef('searchword', $this->getModel()->getState('searchword'));
		$this->assignRef('lists', $lists);
		$this->assignRef('orders', $orders);
		$this->assignRef('items', $rows);
		$this->assignRef('option', $this->getModel()->getState('option'));
		$this->assignRef('dates', $dates); 
		
		// Aggiunta toolbar
		$this->addDisplayToolbar();
		
		// Output del template
		parent::display();
	}

	/**
	 * Mostra la visualizzazione dettaglio del record singolo
	 * @param Object& $row
	 * @access public
	 */
	public function showEntity(&$row) {
		// Aggiunta toolbar
		$this->addShowEntityToolbar();
		
		$doc = JFactory::getDocument ();
		$doc->addStylesheet ( JURI::root ( true ) . '/administrator/components/com_jfbchat/css/generic.css' );
		
		$this->assignRef('option', $this->getModel()->getState('option'));
		$this->assignRef('record', $row);
		
		$this->setLayout('messages');
		parent::display('details');
	}
	 
	/**
	 * Effettua l'output view del file in attachment al browser
	 * 
	 * @access public
	 * @param string $contents
	 * @param int $size
	 * @param array& $fieldsFunctionTransformation
	 * @return void
	 */
	public function sendCSVMessages($data, &$fieldsFunctionTransformation) {
		$delimiter = ';';
		$enclosure = '"';
		// Clean dirty buffer
		ob_end_clean();
		// Open buffer
		ob_start();
		// Open out stream
		$outstream = fopen("php://output", "w");
		// Funzione di scrittura nell'output stream
		function __outputCSV(&$vals, $key, $userData) {
			// Fields value transformations 
			if(isset($vals[INDEX_SENT]) && (int)$vals[INDEX_SENT]) {
				$vals[INDEX_SENT] = date('Y-m-d H:i:s', $vals[INDEX_SENT]);
			}
			if(isset($vals[INDEX_READ]) && (int)$vals[INDEX_READ]) {
				$vals[INDEX_READ] = $vals[INDEX_READ] == 1 ? JText::_('YESREAD') : JText::_('NOREAD');
			}
			if(isset($vals[INDEX_MESSAGE])) {
				$vals[INDEX_MESSAGE] = preg_replace('/<img(.)*\/>/iU', '', $vals[INDEX_MESSAGE]);
			}
			fputcsv($userData[0], $vals, $userData[1], $userData[2]); // add parameters if you want
		}
		// Echo delle intestazioni
		__outputCSV($fieldsFunctionTransformation, null, array($outstream, $delimiter, $enclosure));
		// Output di tutti i records
		array_walk($data, "__outputCSV", array($outstream, $delimiter, $enclosure));
		fclose($outstream);
		// Recupero output buffer content
		$contents = ob_get_clean();
		$size = strlen($contents);
		
		
		header ( 'Pragma: public' );
		header ( 'Cache-Control: must-revalidate, post-check=0, pre-check=0' );
		header ( 'Expires: ' . gmdate ( 'D, d M Y H:i:s' ) . ' GMT' );
		header ( 'Content-Disposition: attachment; filename="users.csv"' );
		header ( 'Content-Type: text/plain' );
		header ( "Content-Length: " . $size );
		echo $contents;
			
		exit ();
	}
}
?>