<?php
//namespace components\com_jfbchat\models; 
/** 
 * @package FBCHAT::AGENTS::components::com_jfbchat
 * @subpackage models
 * @author 2Punti - Marco Biagioni   
 * @copyright (C) 2012 - 2PUNTI SRL
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html  
 */
defined( '_JEXEC' ) or die( 'Restricted access' );

/**
 * Insieme delle responsibilities
 * @package FBCHAT::AGENTS::components::com_jfbchat
 * @subpackage models
 * @since 1.0
 */ 
interface IAgents {
	/**
	 * Restituisce i customers groups impostati
	 * @access public
	 * @return string
	 */
	public function getCustomersGroups(); 
	/**
	 * Controlla se l'utente connesso o non connesso rientra in
	 * uno dei gruppi di sistema ACL impostato come gruppo di
	 * cui considerare gli utenti a livello di customers
	 * @access public
	 * @return boolean
	 */
	public function isValidCustomer();
	
	/**
	 * Controlla se l'utente connesso ha l'id contenuto tra gli utenti
	 * designati come agents del sistema a cui si assegneranno i customers
	 * @access public
	 * @return boolean 
	 */
	public function isValidAgent();
	
	/**
	 * Effettua il vero e proprio assegnamento nella #__fbchat_session_relations
	 * incapsulando l'algoritmo di load balancing di assegnamento customers->agents
	 * @access public
	 * @return boolean 
	 */
	public function assignAgent();  
}

/**
 * Concrete implementation - gestisce relazioni tra customers e agents
 * @package FBCHAT::AGENTS::components::com_jfbchat
 * @subpackage models
 * @since 1.0
 */ 
class jfbcAgents implements IAgents {
	
	/**
	 * User Session object
	 * @access private
	 * @var Object&
	 */
	private $session;
	
	/**
	 * Rappresenta il max life time degli utenti in generale da config
	 * e in particolare anche per gli agents
	 * @access private
	 * @var int
	 */
	private $maxLifetimeAgent;
	 
	/**
	 * Contiene gli IDs corrispondenti ai gruppi ACL designati
	 * come quelli a contenere validi customers
	 * @access private
	 * @var array
	 */
	private $customerGroups;
	
	/**
	 * Rappresenta l'agent id determinato dall'algoritmo di assegnamento 
	 * in base al load balancing
	 * @access private
	 * @var int
	 */
	private $targetAgent;
	
	/**
	 * Database abstraction layer connector
	 * @access protected
	 * @var Object&
	 */
	protected $DBO;
	
	/**
	 * Configuration object reference
	 * @access protected
	 * @var Object&
	 */
	protected $config;
	
	/**
	 * Array che contiene gli IDs degli agents impostati
	 * nella tabella #__fbchat_agents
	 * @access protected
	 * @var array
	 */
	protected $settedAgents;
	
	/**
	 * Restituisce i customers groups impostati per la DB IN query
	 * @access public 
	 * @return string
	 */
	public function getCustomersGroups() {
		$stringResult = null;
		if(is_array($this->customerGroups)) {
			$stringResult = implode (',', $this->customerGroups);
		}
		
		return $stringResult;
	}

	/**
	 * Controlla se l'utente connesso o non connesso rientra in
	 * uno dei gruppi di sistema ACL impostato come gruppo di
	 * cui considerare gli utenti a livello di customers e non  un agent
	 * @access public
	 * @return boolean
	 */
	public function isValidCustomer() {
		// ACL Group ID utente che effettua la http request
		$query = "SELECT group_id" .
				 "\n FROM #__user_usergroup_map" .
				 "\n WHERE " . $this->DBO->quoteName('user_id') .
				 " = " . (int)$this->session->userid;
		$this->DBO->setQuery($query);  
		$userGid = $this->DBO->loadResult();
		// Fallback per utenti guest su gruppo 1 Public
		if(is_null($userGid)) {
			$userGid = 1;
		}
		
		// ID utente se utente connesso
		$usersID = $this->session->userid;
		
		$isInGroups = in_array($userGid, $this->customerGroups);
		$isAgent = in_array($usersID, $this->settedAgents);
		$isValid = (bool) ($isInGroups && !$isAgent);
		return $isValid;
	}
	
	/**
	 * Controlla se l'utente connesso ha l'id contenuto tra gli utenti
	 * designati come agents del sistema a cui si assegneranno i customers
	 * @access public
	 * @return boolean 
	 */
	public function isValidAgent() { 
		$usersID = $this->session->userid;
		$isAgent = in_array($usersID, $this->settedAgents);
		 
		return $isAgent;
	}
	
	/**
	 * Effettua il vero e proprio assegnamento nella #__fbchat_session_relations
	 * incapsulando l'algoritmo di load balancing di assegnamento customers->agents
	 * @access public
	 * @return mixed false se l'assegnamento non ha avuto luogo, un array contenente gli IDs relazionati altrimenti
	 */
	public function assignAgent() {
		$hasAlreadyAgent = false;
		// Se l'utenza appartiene a gruppi customers e non  un agent
		if(!$this->isValidCustomer()){
			return false;	
		}
		// Se il customer ha gi un agent valido assegnato allora nessun assegnamento 
		$query = "SELECT rel.session_id_agent" .
				 "\n FROM #__session AS sess" .
				 "\n INNER JOIN #__fbchat_session_relations AS rel" .
				 "\n ON sess.session_id = rel.session_id_agent" .
				 "\n LEFT JOIN #__fbchat_status AS stat" .
				 "\n ON stat.session_id = sess.session_id" .
				 "\n WHERE rel.session_id_customer = " . $this->DBO->Quote($this->session->session_id) .
				 "\n AND sess.time > " . (int)(time()-$this->maxLifetimeAgent).
				 "\n AND sess.client_id = 0" .
				 "\n AND (stat.status IS NULL OR stat.status != 'invisible')";
		$this->DBO->setQuery($query);
		$hasAlreadyAgent = (bool)$this->DBO->loadResult();
		if($hasAlreadyAgent) {
			return false;
		}
			
		// Assegnamento necessario: Load balancing logic query
		$query = "SELECT sess.session_id, COUNT(rel.session_id_customer) AS numcustomers" .
				 "\n FROM #__session AS sess" .
				 "\n LEFT JOIN #__fbchat_session_relations AS rel" .
				 "\n ON rel.session_id_agent = sess.session_id" .
				 "\n LEFT JOIN #__fbchat_status AS stat" .
				 "\n ON stat.session_id = sess.session_id" .
				 "\n WHERE sess.userid IN (" . implode(',', $this->settedAgents) . ")" .
				 "\n AND sess.time > ". (int)(time()-$this->maxLifetimeAgent) .
				 "\n AND sess.client_id = 0" .
				 "\n AND (stat.status IS NULL OR stat.status != 'invisible')" .
				 "\n GROUP BY sess.session_id";
		$this->DBO->setQuery($query);
		$results = $this->DBO->loadObjectList();
		if(is_array($results) && count($results)) {
			function user_defined_reduce($A, $B){ 
			   if(!is_null($A)) {
					$minfound =  $A->numcustomers < $B->numcustomers ? $A : $B;
					$A = $minfound; 
				} else {
					$A = $B;
				}
				return $A;
			}
			$moreFreeAgent = array_reduce($results,"user_defined_reduce");
			$this->targetAgent = $moreFreeAgent->session_id;
			// Se  stato determinato un agent valido si provvede all'assegnamento
			if(!isset($this->targetAgent)){
				return false;
			}
			$query = "INSERT INTO #__fbchat_session_relations " .
					 "\n VALUES ( " . $this->DBO->Quote($this->session->session_id) . "," .
					 "\n " . $this->DBO->Quote($this->targetAgent) . " )" .
					 "\n ON DUPLICATE KEY UPDATE session_id_agent = " . $this->DBO->Quote($this->targetAgent);
			$this->DBO->setQuery($query);
			if(!$this->DBO->execute()){
				return false;
			}  
			// Assegnamento avvenuto con successo
			return array($this->session->session_id, $this->targetAgent);
		} else {
			return false;
		} 
	}
 
	/**
	 * Class constructor
	 * @access public
	 * @return Object& 
	 */
	public function __construct(&$session) {
		// User session reference
		$this->DBO = JFactory::getDBO();
		$this->session = &$session;
		$this->config = JComponentHelper::getParams ( 'com_jfbchat' );
		$this->customerGroups = $this->config->get('customer_gids');
		$this->maxLifetimeAgent = $this->config->get('maxlifetime_session'); 
		// Setting degli utenti agents designati
		$query = "SELECT userid FROM #__fbchat_agents";
		$this->DBO->setQuery($query);
		$this->settedAgents = $this->DBO->loadColumn();
	}
}
?>