<?php
//namespace components\com_jfbchat\models; 
/** 
 * @package FBCHAT::AVATAR::components::com_jfbchat
 * @subpackage models
 * @author 2Punti - Marco Biagioni    
 * @copyright (C) 2012 - 2PUNTI SRL
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html  
 */
defined( '_JEXEC' ) or die( 'Restricted access' );

/**
 * Permette la gestione degli avatar personalizzati permettendone l'upload e la cancellazione,
 * nonch la visualizzazione renderizzata in un iframe popup da javascript side 
 * @package FBCHAT::AVATAR::components::com_jfbchat
 * @subpackage models
 * @since 1.0
 */ 
class jfbcAvatar {
	/**
	 * DB object connector
	 * @access private
	 * @var Object
	 */
	private $DBO;
	
	/**
	 * Site URL
	 * @access private
	 * @var string
	 */
	private $siteurl;
	
	/**
	 * Plugin configuration object
	 * @access private
	 * @var Object
	 */
	private $config;
	
	/**
	 * Session object
	 * @access private
	 * @var Object&
	 */
	private $session;
	
	/**
	 * Percorso folder per lo storage dei file avatar
	 * @access private
	 * @var string
	 */
	private $avatarFolder;
	
	/**
	 * nome file immagine no avatar available
	 * @access private
	 * @var string
	 */
	private $noAvatarImgName;

	/**
	 * Formato standard avatar forced
	 * @access private
	 * @static
	 * @var string
	 */
	private static $avatarFormat = 'png';
	
	/**
	 * Subpath avatar files
	 * @access private
	 * @static
	 * @var string
	 */
	private static $avatarSubPath = '/images/avatars/';
	  
	/**
	 * Crea un nome codificato con md5 a partire dall'ID utente
	 * per il file immagine caricato rinominandolo
	 * @access private 
	 * @param string $extension2Append
	 * @return array Il path completo e soltanto il filename al file thumb avatar creato o da creare
	 */
	private function createAvatarThumbnailFileName($extension2Append = 'png') {
		// Calculate in base all'md5 di id utente e username
		$calculatedHash = md5($this->session->session_id);
		$finalName = $calculatedHash . '.' . $extension2Append;
		$completePathName = $this->avatarFolder . '/' . $finalName;
		$liveUrl = $this->siteurl . '/images/avatars';
		 
		return array($completePathName, $finalName, $liveUrl);
	}
	
	/**
	 * Genere l'immagine thumbnail dell'avatar inviato
	 * @access private
	 * @param string $originalFilename
	 * @param string $thumbFilename
	 * @return boolean 
	 */
	private function createThumbnail($originalFilename, $thumbFilename) { 
		// Require delle libreria phpthumb in lazy loading
		require_once JPATH_COMPONENT . '/libraries/phpthumb/thumb.factory.php';
		$thumb = PhpThumbFactory::create($originalFilename);
		// Selezione modo di resizing
		switch ($this->config->get('cropmode')){
			case '0':
				$thumb->resize(50, 50);
				break;
				
			case '1':
				$thumb->adaptiveResize(50, 50);
				break;
		}
		
		$thumb->save($thumbFilename, 'png');
	}

	/**
	 * Mostra i due form utente, uno per l'upload del file immagine e
	 * l'altro per la visualizzazione/cancellazione avatar attuale
	 * Permette anche di visualizzare i messaggi informativi all'utente
	 * venendo richiamato dalle altre funzioni pubbliche della classe
	 * @access public
	 * @param string $msg Eventuale messaggio informativo utente
	 * @return void 
	 */
	public function showForms($msg = null) {
		// Controlla se l'avatar  stato caricato per l'utente corrente
		$avatar = $this->createAvatarThumbnailFileName();
		$base = JURI::base(); 
		if(file_exists($avatar[0])) {
			$userAvatar = $avatar[1] .'?nocache='.time();
		} else {
			$userAvatar = $this->noAvatarImgName .'?nocache='.time();
		}
		// Include css file da template
		$tmplName = $this->config->get('template');
		$language = $this->config->get('language');
		$includes = <<<INCLUDES
		<script type="text/javascript" src="{$this->siteurl}/languages/$language" charset="utf-8"></script>
		<script type="text/javascript" src="{$baseUri}components/com_jfbchat/js/jquery.js"></script>
		<script> 
		var jfbc_baseURI='$base';
		// Inject per IE7/IE8 Fix
        if($.browser.msie && $.browser.version <= 7){
            $('<link/>').attr({'href':jfbc_baseURI + 'components/com_jfbchat/css/fix/ie7fix.css', 'rel':'stylesheet', 'type':'text/css'}).appendTo('head');   
        };
        
		// Inject per IE7/IE8 Fix
        if($.browser.msie && $.browser.version <= 8){
            $('<link/>').attr({'href':jfbc_baseURI + 'components/com_jfbchat/css/fix/ie8fix.css', 'rel':'stylesheet', 'type':'text/css'}).appendTo('head');   
        };
		</script>
		<link type="text/css" rel="stylesheet" media="all" href="{$this->siteurl}/css/$tmplName" />
		<!--[if lt IE 9]>
			<style type="text/css">
				div.up {
					margin-top: 0px; 
				}
			</style>
		<![endif]-->
INCLUDES;

		$deleteForm = <<<DELETEFORM
		<form name="deleteform" enctype="multipart/form-data" id="deleteform" method="post" action="index.php">
			<fieldset><legend><script>document.write(jfbc_user_avatar);</script></legend>
			    <img class="avatar_img" src="{$this->siteurl}/images/avatars/$userAvatar" alt="avatar" />
				<div class="formbutton cancel">
					<input id="avatar_delete" type="submit" value="" />
			    	<script>$('#avatar_delete').val(jfbc_avatar_delete);</script>
					<input type="hidden" name="option" value="com_jfbchat" />
					<input type="hidden" name="controller" value="avatar" />
					<input type="hidden" name="task" value="deleteAvatar" />
					<input type="hidden" name="format" value="raw" />
				</div>
			</fieldset>
		</form>
DELETEFORM;
		
		$uploadForm = <<<UPLOADFORM
		<form name="uploadform" enctype="multipart/form-data" id="uploadform" method="post" action="index.php">
			<fieldset><legend><script>document.write(jfbc_upload_newavatar);</script></legend> 
				<input type="file" name="newavatar" />
				<div class="formbutton up">
					<input id="avatar_upload" type="submit" value="" /> 
					<script>$('#avatar_upload').val(jfbc_upload);</script>
					<input type="hidden" name="option" value="com_jfbchat" />
					<input type="hidden" name="controller" value="avatar" />
					<input type="hidden" name="task" value="doUpload" />
					<input type="hidden" name="format" value="raw" />
				</div>
			</fieldset>
		</form> 
UPLOADFORM;
		
		
		echo $includes;
		// Controllo presenza GD library
		if (extension_loaded('gd') && function_exists('gd_info')) { 
			echo $deleteForm;
			echo $uploadForm;
		} else {
			$msg = '<script>document.write(jfbc_gderror);</script>';
		}
	
		$userMessage = <<<MESSAGE
		<div class="upload_usermessage">$msg</div>
MESSAGE;
		echo $userMessage;
	}

	/**
	 * Effettua l'upload del file avatar
	 * @access public
	 * @return void 
	 */
	public function doUpload() {
		// Recupera il file in upload
		$tmpFile = $_FILES['newavatar']['tmp_name'];
		$tmpFileName = $_FILES['newavatar']['name'];
		 
		// Controlla se il file  inferiore alla dimensione massima, altrimenti interrompe con errore
		$tmpFileSize = $_FILES['newavatar']['size'];
		$allowedFileSize = $this->config->get('maxfilesize') * 1024 * 1024; // MB->Bytes
		if($tmpFileSize > $allowedFileSize) {
			$msg = "<script>document.write(jfbc_size_error);</script> Max {$this->config->get('maxfilesize')} MB.";
			$this->showForms($msg);
			return;
		}
		
		// Controlla se il file ha una estensione ammessa, altrimenti interrompe con errore
		$allowedExtensions = explode(',', $this->config->get('avatar_allowed_extensions')); 
		$tmpFileExtension = array_pop(explode('.', $tmpFileName));
		if(!in_array($tmpFileExtension, $allowedExtensions)) {
			$msg = "<script>document.write(jfbc_ext_error);</script>{$this->config->get('avatar_allowed_extensions')}.";
			$this->showForms($msg);
			return;
		}
				
		// Controlla se la cartella target  scrivibile, altrimenti prova a settare i chmod, altrimenti interrompe con errore
		if(!is_writable($this->avatarFolder)) {
			// prova a amodificare i permessi
			try {
				if(!chmod($this->avatarFolder, 0775)) {
					throw new Exception('<script>document.write(jfbc_dir_writable);</script>');
				}
			} catch(Exception $e) {
				$msg = $e->getMessage();
				$this->showForms($msg);
				return;
			}
		}
		 
		// Effettua la move uploaded file
		if(!move_uploaded_file($tmpFile, $this->avatarFolder . '/' . $tmpFileName)) {
			$msg = "<script>document.write(jfbc_upload_error);</script>";
			$this->showForms($msg);
			return;
		}
		
		// Richiede la creazione di un hash filename per il thumbnail con format force a png
		$thumbnailFileName = $this->createAvatarThumbnailFileName(self::$avatarFormat);
		
		// Se esiste il file ripuliamo adesso; il file name  sempre univoco per utente anche il formato  forced a png
		if(file_exists($thumbnailFileName[0])) {
			unlink($thumbnailFileName[0]);
		}
		
		// Genera un thumbnail per l'immagine caricata
		$this->createThumbnail($this->avatarFolder . '/' . $tmpFileName, $thumbnailFileName[0]);
		
		// Elimina il file originale caricato 
		unlink ($this->avatarFolder . '/' . $tmpFileName);
		 
		// Richiama la showForms con user message 
		$msg = '<script>document.write(jfbc_success_avatar);</script>';
		$this->showForms($msg);
	}

	/**
	 * Cancella il file avatar attuale
	 * @access public
	 * @return void 
	 */
	public function deleteAvatar() {
	// Richiede la creazione di un hash filename per il thumbnail con format force a png
		$thumbnailFileName = $this->createAvatarThumbnailFileName(self::$avatarFormat);
		
		// Se esiste il file ripuliamo adesso; il file name  sempre univoco per utente anche il formato  forced a png
		if(file_exists($thumbnailFileName[0])) {
			if(unlink($thumbnailFileName[0])){ 
				$msg = '<script>document.write(jfbc_success_delete_avatar);</script>'; 
			} else {
				$msg = '<script>document.write(jfbc_error_deleting_avatar);</script>'; 
			}
		} else {
			$msg = '<script>document.write(jfbc_avatar_notfound);</script>';
		}
		
		// Richiama la showForms con user message  
		$this->showForms($msg);
	}
	
	/**
	 * Restituisce staticamente il nome del file avatar per l'utente corrente se presente
	 * Viene utilizzato a scopo frontend chat utilities 
	 * @param string $data
	 * @return string
	 */
	public static function getAvatar($data) { 
		// Calculate in base all'md5 user session ID
		$calculatedHash = md5($data);
		$finalName = $calculatedHash . '.' . self::$avatarFormat; 
		$filePath = JPATH_COMPONENT . self::$avatarSubPath . $finalName;
		$liveUrl = JURI::base() . '/components/com_jfbchat/images/avatars/' . $finalName . '?nocache='.time();
		
		// Controlliamo se esiste un filename avatar utente altrimenti fallback empty image
		if(file_exists($filePath)) {
			return $liveUrl;
		} else { 
			return 'http://www.gravatar.com/avatar/'.$calculatedHash.'?d=wavatar&s=80';
		}  
	}

	/**
	 * Class constructor
	 * @access public 
	 * @param Object& $session
	 * @return Object &
	 */
	public function __construct(&$session) {
		$this->DBO = &JFactory::getDBO();
		$this->siteurl = JURI::base() . '/components/com_jfbchat/';
		$this->avatarFolder = JPATH_COMPONENT . '/images/avatars';
		$this->session = &$session;
	 
		// CONFIG LOAD
		$this->config = &JComponentHelper::getParams('com_jfbchat'); 
		$this->noAvatarImgName = 'no-avatar.png';
	}
} 
?>