<?php
//namespace components\com_jfbchat\models; 
/** 
 * @package FBCHAT::EXPORT::components::com_jfbchat
 * @subpackage models
 * @author 2Punti - Marco Biagioni
 * @version $Id: export.php 1 16/03/2012 16:31:20Z marco $     
 * @copyright (C) 2012 - 2PUNTI SRL
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html  
 */

/**
 * Permette l'esportazione delle proprie conversazioni mantenute in sessione
 * dopo lo svolgimento delle stesse recuperate da ID utente
 * @package FBCHAT::EXPORT::components::com_jfbchat
 * @subpackage models
 * @since 1.0
 */ 
class jfbcExport {
	/**
	 * User object connesso
	 * @access private
	 * @var string
	 */
	private $meUser;
	/**
	 * User object connesso
	 * @access private
	 * @var string
	 */
	private $otherUser; 
	
	/**
	 * ID della session conversation da esportare
	 * @access private
	 * @var Object&
	 */
	private $userConversation;
	
	/**
	 * SESSION array reference
	 * @access private
	 * @var array&
	 */
	private $session;
	
	/**
	 * Rimuove parti di non interesse e purifica i messaggi scambiati per
	 * poter esportare dati corretti
	 * @access private
	 * @param string $message
	 * @return string
	 */
	private function purifyMessage($message) {
		// Strip delle immagini delle emeoticons con estrazione battitura   
		$message = preg_replace ('/(<img)\s(alt=")(.+)(")*(title=")(.+)(")(.*\/>)/iUu', "$6", $message);
		
		// Strip html tags
		$message = strip_tags($message);
		
		return $message;
	}
	
	/**
	 * Generatore del file di testo contentente la conversazione
	 * @access public
	 * @return void
	 */
	public function exportFile() {
		// Recuperiamo prima i dati della conversazione apponendo i relativi sender
		$conversation = $this->session['jfbcchat_user_' . $this->userConversation];
		$exportConversationString = '';
		if(is_array($conversation)) {
			foreach ($conversation as $message) {
				// Decisione sul contenuto del messaggio
				switch (@$message['type']){
					case 'file':
						$renderedMessage = 'FILE[' . $message['message'] . ']';
						break;
						
					case 'message':
					default:
						$renderedMessage = $this->purifyMessage($message['message']);
						break;
				}
				// Decisione sul sender del messaggio
				if(!(bool)$message['self']) {
					$sender = $this->otherUser;
				} else {
					$sender = $this->meUser;
				}
				$exportConversationString .= $sender . ": " . $renderedMessage . PHP_EOL;
			}
		}
		
		// Export file txt
		$cont_dis = 'attachment';
		$mimeType = 'text/plain';
		$filename = $this->meUser. '_' . $this->otherUser . '_' . date('Y-m-d') . '.txt';
		// required for IE, otherwise Content-disposition is ignored
		if (ini_get ( 'zlib.output_compression' )) {
			ini_set ( 'zlib.output_compression', 'Off' );
		} 
		$size = strlen($exportConversationString);	
	 	$mod_date = date ( 'r' ); 
	 	//Output del file 
	 	header ( "Pragma: public" );
		header ( "Cache-Control: must-revalidate, post-check=0, pre-check=0" );
		header ( "Expires: 0" ); 
		header ( 'Content-Disposition:' . $cont_dis . ';' . ' filename="' . $filename . '";' . ' modification-date="' . $mod_date . '";' . ' size=' . $size . ';' ); //RFC2183
		header ( "Content-Type: " . $mimeType ); // MIME type
		header ( "Content-Length: " . $size ); 
	 	echo $exportConversationString;
	 	exit();
	}
	
	/**
	 * Class constructor
	 * @access public
	 * @param int $useridChat
	 * @param string $usernameChat
	 * @param string $meUserName
	 * @return Obejct&
	 */
	public function __construct($useridChat, $otherUserName, $meUserName) {
		$this->session = &$_SESSION;
		$this->userConversation = $useridChat;
		$this->meUser = $meUserName;
		$this->otherUser = $otherUserName; 
	}
}