<?php
//namespace components\com_jfbchat\models; 
/** 
 * @package FBCHAT::components::com_jfbchat
 * @subpackage models
 * @author 2Punti - Marco Biagioni
 * @version $Id: garbage.php 43 17/04/2012 16:31:20Z marco $     
 * @copyright (C) 2012 - 2PUNTI SRL
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html  
 */
defined ( '_JEXEC' ) or die ( 'Restricted access' );

/**
 * Responsabilit classe jfbcGarbage
 * @package FBCHAT::components::com_jfbchat
 * @subpackage models
 * @since 1.0
 */
interface IGarbage {
	/**
	 * Esegue il garbage collector process dando avvio solo se il calcolo probabilistico ha esito positivo
	 * @access public
	 * @return Boolean
	 */
	public function execGC();
}

/**
 * Classe che gestisce il garbage collector dei messaggi obsoleti nel database 
 * @package FBCHAT::components::com_jfbchat
 * @subpackage models 
 * @since 1.0
 */
class jfbcGarbage implements IGarbage{
	
	/**
	 * Memorizza se il garbage  attivato 
	 * @access private
	 * @var Boolean
	 */
	private $enabled;
	/**
	 * Rappresenta il massimo tempo oltre cui considerare un messaggio obsoleto 
	 * @access private
	 * @var int
	 */
	private $maxLifeTime; 
	/**
	 * Rappresenta il massimo tempo oltre cui considerare un file obsoleto nella cache 
	 * @access private
	 * @var int
	 */
	private $maxLifeTimeFile;
	/**
	 * Rappresenta il massimo tempo oltre cui considerare un utente come non pi online per le statistiche realtime
	 * @access private
	 * @var int
	 */
	private $maxRealstatsTime; 
	/**
	 * Path della cache folder per i files
	 * @access private
	 * @var int
	 */
	private $cacheFolder;
	/**
	 * Decide la probabilit che il garbage collector venga avviato
	 * @access private
	 * @var int
	 */
	private $probability;
	/**
	 * @property Boolean $divisor - Il divisore probabilistico 
	 * @access private
	 * @var int
	 */
	private $divisor;
	/**
	 * Memorizza un reference all'oggetto database
	 * @access private
	 * @var Object &
	 */
	private $DBO;
	
	/**
	 * Esegue il calcolo probabilistico vero e proprio
	 * @access private
	 * @return Boolean
	 */
	private function probabilityFn() {
		$randomNumber = rand ( 0, $this->divisor );
		if (0 < $randomNumber && $randomNumber <= $this->probability) {
			return true;
		} else {
			return false;
		}
	}
	
	/**
	 * Costruisce la query di DELETE dal DB in caso di match
	 * @access private
	 * @return String
	 */
	private function buildQuery() {
		$queries = array();
		$time_attuale = time ();
		$soglia = $time_attuale - $this->maxLifeTime;
		$sogliaRealtime = $time_attuale - $this->maxRealstatsTime;
		 
		//Assicuriamoci di cancellare messsaggi letti
		$queries['private'] = "DELETE FROM #__fbchat WHERE (" . $this->DBO->quoteName ( 'sent' ) .
							  "< " . $this->DBO->Quote ( $soglia ) .
							  " AND " . $this->DBO->quoteName ( 'read' ) . " = 1)";
		
		//Assicuriamoci di cancellare gli status records non pi validi in sessione per joomla
		$queries['status'] = "DELETE FROM #__fbchat_status WHERE " . $this->DBO->quoteName ( 'session_id' ) .
							 "\n NOT IN ( SELECT " . $this->DBO->quoteName ( 'session_id' ) . " FROM #__session" .
							 "\n WHERE " . $this->DBO->quoteName ( 'client_id' ) . " = 0 )";
		
		//Assicuriamoci di cancellare gli status records non pi validi in sessione per joomla
		$queries['realtime'] = "DELETE FROM #__fbchat_realstats WHERE " . $this->DBO->quoteName ( 'lastupdate_time' ) .
							   "< " . $this->DBO->Quote ( $sogliaRealtime );
 
		return $queries;
	}
	  
	/**
	 * Esegue il garbage collector process dando avvio solo se il calcolo probabilistico ha esito positivo 
	 * @access public
	 * @return Boolean
	 */
	public function execGC() {
		$match = $this->probabilityFn ();
		if ($match && $this->enabled) {
			// Garbage messaggi su DB
			$queries = $this->buildQuery ();
			
			// Garbage private messages 
			$this->DBO->setQuery ( $queries['private'] );
			if (! $this->DBO->execute ()) {
				$errorMsg = $this->DBO->getErrorMsg ();
				return $errorMsg;
			} 
			
			// Garbage status obsoleti da joomla
			$this->DBO->setQuery ( $queries['status'] );
			if (! $this->DBO->execute ()) {
				$errorMsg = $this->DBO->getErrorMsg ();
				return $errorMsg;
			}
		 
			// Garbage status session id records realtime obsoleti
			$this->DBO->setQuery ( $queries['realtime'] );
			if (! $this->DBO->execute ()) {
				$errorMsg = $this->DBO->getErrorMsg ();
				return $errorMsg;
			}
			
			// Garbage files cache folder
	   		if(is_dir($this->cacheFolder)) {
			    $filenames = array();
			    if(class_exists('DirectoryIterator', false)) {
			    $iterator = new DirectoryIterator($this->cacheFolder);
				    foreach ($iterator as $fileinfo) {
				        if ($fileinfo->isFile()) { 
				        	if(time() - $fileinfo->getMTime() > $this->maxLifeTimeFile) {
				        		 unlink($fileinfo->getRealPath());
				        	} 
				        }
				    }   
			    }
	   		}
			return true;
		} else {
			return false;
		}
	}
	
	/**
	 * Setta i parametri di configurazione nelle private properties 
	 * @access public
	 * @return Object&
	 */
	public function __construct() {
		/** 
		 * Inizializzazione oggetto garbage collector 
		 */
		$this->DBO = JFactory::getDBO ();
		//Get dei parametri del componente
		$configParams = JComponentHelper::getParams ( 'com_jfbchat' );
		$this->probability = ( int ) $configParams->get ( 'probability' );
		$this->maxLifeTime = ( int ) $configParams->get ( 'maxlifetime' ) * (24 * 60 * 60);
		$this->maxLifeTimeFile = ( int ) $configParams->get ( 'maxlifetime_file' ) * (24 * 60 * 60);
		$this->maxRealstatsTime = ( int ) $configParams->get ( 'maxlifetime_session' );
	 	$this->enabled = ( int ) $configParams->get ( 'gcenabled' );
		$this->divisor = 100;
		$this->cacheFolder = 'components/com_jfbchat/cache/';
	}
} 