<?php
//namespace components\com_jfbchat\models;
/**  
 * Gestore dei messaggi e dei dati 
 * @package FBCHAT::REALSTATS::components::com_jfbchat 
 * @subpackage models
 * @author 2Punti - Marco Biagioni   
 * @copyright (C) 2012 - 2PUNTI SRL
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html  
 */ 
defined ( '_JEXEC' ) or die ( 'Restricted access' ); 

/**
 * Responsabilit classe gestore dei messaggi e dei dati  
 * @package FBCHAT::REALSTATS::components::com_jfbchat
 * @subpackage models 
 * @since 1.0
 */
interface IRealstats { 
	/**
	 * Si occupa di aggiornare il record contestuale alla sessione utente connesso
	 * nella tabella 1 a 1 #__fbchat_realstats -> #__session
	 * @access public
	 * @return void
	 */
	public function refreshRealtime();
}

/**
 * Realstats class frontend implementation 
 * @package FBCHAT::REALSTATS::components::com_jfbchat
 * @subpackage models 
 * @since 1.0
 */
class jfbcRealstats implements IRealstats { 
	/**
	 * DB Connector
	 * @var Object&
	 * @access private
	 */
	private $DBO;
	
	/**
	 * Session ID utente in refresh
	 * @var Object&
	 * @access private
	 */
	private $session;

	/**
	 * Ottiene dalla POST AJAX il current URL dove si trova l'utente
	 * @access protected
	 * @return string 
	 */
	protected function getNowPage() {
		return urldecode(JRequest::getVar('nowpage', null, 'POST'));
	}

	/**
	 * Ottiene dalla POST AJAX il current URL dove si trova l'utente
	 * @access protected
	 * @return string
	 */
	protected function getCurrentName() {
		return JRequest::getVar('currentname', null, 'POST');
	}
	
	/**
	 * Valuta se il tracking realstats  abilitato
	 * @access protected
	 * @return string
	 */
	protected function getTrackingIsOff() {
		$nowPagePosted = $this->getNowPage();
		if(preg_match('/notrack/i', $nowPagePosted)) {
			return true;
		}
	}
	
	/**
	 * Si occupa di aggiornare il record contestuale alla sessione utente connesso
	 * nella tabella 1 a 1 #__fbchat_realstats -> #__session
	 * @access public
	 * @return boolean Unit testing Use
	 */
	public function refreshRealtime() {
		// Se non  abilitato il tracking non si effettua nessuna operazione
		if($this->getTrackingIsOff()) {
			return false;
		}
		
		// Ottenimento pagina utente corrente
		$nowPage = $this->getNowPage();
		$currentName = $this->getCurrentName();
		$field = null;
		$value = null;
		$update = null;
		if($currentName) {
			$field = ", \n " . $this->DBO->quoteName('current_name');
			$value =  "," . $this->DBO->Quote($currentName);
			$update = ", \n " . $this->DBO->quoteName('current_name') . "=" .  $this->DBO->Quote($currentName);
		}
		
		// Build query insert/update
		$insertUpdateQuery = "INSERT INTO #__fbchat_realstats" .
							 "\n ( " . $this->DBO->quoteName('session_id_person') . "," . 
							 "\n " . $this->DBO->quoteName('nowpage') . "," .
							 "\n " . $this->DBO->quoteName('lastupdate_time') . $field . " )" . 
							 "\n VALUES ( " . $this->DBO->Quote($this->session->session_id) . "," .
							 $this->DBO->Quote($nowPage) . "," .
							 $this->DBO->Quote(time()) .  $value . " )" .
							 "\n ON DUPLICATE KEY UPDATE" .
							 "\n " . $this->DBO->quoteName('nowpage') . " = " . $this->DBO->Quote($nowPage) . "," .
							 "\n " . $this->DBO->quoteName('lastupdate_time') . "=" .  $this->DBO->Quote(time()) . $update;
		$this->DBO->setQuery($insertUpdateQuery);
		if(!$this->DBO->execute()) {
			return false;
		}
		return true;
	}

	/**
	 * Class constructor
	 * @access public
	 * @return Object& 
	 */
	public function __construct(&$session) {
		// User session reference
		$this->DBO = JFactory::getDBO();
		$this->session = &$session; 
	}
}