<?php
//namespace components\com_jfbchat\models;
/**  
 * Gestore dei messaggi e dei dati 
 * @package FBCHAT::DATA::components::com_jfbchat 
 * @subpackage models
 * @author 2Punti - Marco Biagioni   
 * @copyright (C) 2012 - 2PUNTI SRL
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html  
 */ 
defined ( '_JEXEC' ) or die ( 'Restricted access' );
define ( 'JFBC_AGENT', 2 );
define ( 'JFBC_CUSTOMER', 1 );
define ( 'JFBC_NOUSER', 0 );

/**
 * Responsabilit classe gestore dei messaggi e dei dati  
 * @package FBCHAT::components::com_jfbchat
 * @subpackage models 
 * @since 1.0
 */
interface IReceiver {

	/**
	 * Execute della app logic da controller
	 * @access public 
	 * @return void
	 */
	public function appDispatch();
}

/**
 * Classe gestore dei messaggi e dei dati  
 * @package FBCHAT::components::com_jfbchat
 * @subpackage models 
 * @since 1.0
 */
class jfbcReceiver implements IReceiver{
	/**
	 * Session Object
	 * @access private
	 * @var Object &
	 */
	private $session;
	
	/**
	 * Me user Object
	 * @access private
	 * @var Object &
	 */
	private $myUser;
	 
	/**
	 * Memorizza un reference all'oggetto database
	 * @access private
	 * @var Object &
	 */
	private $DBO;
	
	/**
	 * Component config
	 * @access private
	 * @var Object &
	 */
	private $config;
	
	/**
	 * Gestore degli agents e degli assegnamenti
	 * @access private
	 * @var Object &
	 */
	private $agentsDirector;
	
	/**
	 * Realstats communication instance
	 * @access private
	 * @var Object &
	 */
	private $realStats;
	
	/**
	 * Serverstats tracking instance
	 * @access private
	 * @var Object &
	 */
	private $serverStats;
	
	/**
	 * Query chunk se gruppo Public selezionato per i valid customers
	 *
	 * @access private
	 * @var array
	 */
	private $queryChunk;
	
	/**
	 * Array dei messaggi da inserire nella response
	 * @access protected
	 * @var array
	 */
	protected $messages;
	
	/**
	 * Array associativo della response HTTP
	 * @access protected
	 * @var array
	 */
	protected $response;
	
	/**
	 * Si occupa di controllare se esistono nuovi MSGFILE presenti nel database con status = 1
	 * che non sono stati refreshati in sessione e li pone nella response['downloads'] memorizzandoli
	 * separatamente in sessione per evitare doppie notifiche e aggiornamenti 
	 * @access private
	 * @return void
	 */
	private function refreshMsgFileSessionStatus() { 
		$query = "SELECT id, " . $this->DBO->quoteName('session_id_to') . " FROM #__fbchat" .
				 "\n WHERE type=" . $this->DBO->Quote('file') .
				 "\n AND status = 1" .
				 "\n AND " . $this->DBO->quoteName('session_id_from') . " = " . $this->DBO->Quote($this->session->session_id);
		$this->DBO->setQuery($query);
		$msgFiles = $this->DBO->loadObjectList();
		
		// Gestiamo il session array downloads gi notificati
		if(!isset($_SESSION['notified_downloads'])) {
			$_SESSION['notified_downloads'] = array();
		} 
		
		if(is_array($msgFiles) && count($msgFiles)) {
			foreach ($msgFiles as $msgFile) {
				if(in_array($msgFile->id, $_SESSION['notified_downloads'])) {
					// Do nulla
				} else {
					$conversation2Refresh = &$_SESSION['jfbcchat_user_' . $msgFile->session_id_to];
					if(is_array($conversation2Refresh) && count($conversation2Refresh)) {
						foreach ($conversation2Refresh as &$singleSessionMessage) {
							if($singleSessionMessage['id'] == $msgFile->id) {
								$singleSessionMessage['status'] = 1;
								$_SESSION['notified_downloads'][] = $msgFile->id;
								$this->response['downloads'][] = array($msgFile->session_id_to, $msgFile->id);
							}
						}
					}
				}
			} 
		}
	}
	
	/**
	 * Effettua un reverse reduce sul'ID di sessione MD5 per arrivare
	 * ad una stringa da appendere al prefix del name assegnato ai guest users
	 * @param string $sessionID
	 * @access private
	 * @return string
	 */
	private function generateRandomGuestNameSuffix($sessionID) {
		// Conversione da base 16 a base 36 ovvero 10 digits + 26 chars. Mantiene 128 bit come l'MD5 ma permette di avere una stringa pi corta
		$hash = base_convert($sessionID, 16, 36);
		
		// Recuperiamo la parte numerica dell'hash in base 36
		$numericHashArray = preg_match_all('/\d/i', $hash, $matches);
		
		if(is_array($matches[0]) && count($matches[0])) {
			$appendHashSuffix = '_' . implode('', $matches[0]);
			// Limitiamo a 4 cifre il numeric suffix
			$appendHashSuffix = $this->config->get('guestprefix') . substr($appendHashSuffix, 0, 5);
		}
		
		return $appendHashSuffix;
	}
	
	/**
	 * Output response application/json
	 * @access private 
	 * @return void
	 */
	private function sendResponse() {
		if (! empty ( $this->messages )) {
			$this->response ['messages'] = $this->messages;
		}
			
		// Fondamentale per far arrivare prima la paramslist ad initialize = 1
		ksort($this->response);
		header ( 'Content-type: application/json; charset=utf-8' );
		echo json_encode ( $this->response );
		exit ();
	}
	
	/**
	 * Ottiene un eventuale overridename valido per utenti guest
	 * @access private
	 * @return string
	 */
	private function getOverrideName($session_id) { 
		$sql = "SELECT override_name FROM #__fbchat_status" .
				"\n WHERE session_id = " . $this->DBO->Quote($session_id);
		$this->DBO->setQuery($sql);
		$overrideName = $this->DBO->loadResult();
		 
		return $overrideName;
	}
	
	/**
	 * Restituisce se impostata la default suggestion da servire alla prima AJAX request initialize
	 * @access private
	 * @return mixed
	 */
	private function getDefaultSuggestion() {
		$sql =  "SELECT " . $this->DBO->quoteName('suggestion') .
				"\n FROM #__fbchat_suggestions" .	
				"\n WHERE " . $this->DBO->quoteName('autoinvite') . " = 1";
		$this->DBO->setQuery($sql);
		$suggestion = $this->DBO->loadResult();
			
		return $suggestion ? $suggestion : false;
	}
	
	/**
	 * Imposta nella response lo status dell'utente
	 * @access protected
	 * @return void
	 */
	protected function getStatus() {  
		$sql = "SELECT message, status, override_name, skypeid, skype_enable FROM #__fbchat_status" .
			   "\n WHERE session_id = " . $this->DBO->Quote($this->session->session_id); 
		$this->DBO->setQuery($sql); 
		$chat = $this->DBO->loadAssoc();  
		
		if (empty($chat['status'])) {
			$chat['status'] = 'available';
		} 
		
		if (empty($chat['message'])) {
			$chat['message'] = "-";
		}
		
		if (empty($chat['override_name'])) {
			$chat['override_name'] = null;
		}
		
		if (empty($chat['skypeid'])) {
			$chat['skypeid'] = null;
		}
	
		if (empty($chat['skype_enable'])) {
			$chat['skype_enable'] = 0;
		}
		
		$status = array('message' => $chat['message'], 
						'status' => $chat['status'], 
						'override_name' => $chat['override_name'],
						'skype_id' => $chat['skypeid'],
						'skype_enable' => $chat['skype_enable']);
		$this->response['userstatus'] = $status;
	}
	 
	/**
	 * Imposta la lista utenti
	 * @access protected 
	 * @return boolean
	 */
	protected function getBuddyList() {
		// Init chunks query
		$select = null;
		$join = null;
		$where = null;
		$distinct = null;
		
		// Prima operazione di recupero rango utenza
		if($this->agentsDirector->isValidAgent()){
			$select = ", stats.nowpage, serverstats.geolocation";
			$distinct = " DISTINCT ";
			$join = "\n INNER JOIN #__fbchat_session_relations AS rel ON rel.session_id_customer = sess.session_id" .
					"\n LEFT JOIN #__fbchat_realstats AS stats ON stats.session_id_person = sess.session_id" .
					"\n LEFT JOIN #__user_usergroup_map AS map ON map.user_id = sess.userid";
			$join .= "\n LEFT JOIN #__fbchat_serverstats AS serverstats ON serverstats.session_id_person = sess.session_id";
			$where = "\n AND rel.session_id_agent = " . $this->DBO->Quote($this->session->session_id) .
					 "\n AND (map.group_id IN (" . $this->agentsDirector->getCustomersGroups() . ")" . $this->queryChunk . ")";
			// Settaggio rango utente
			$this->rango = JFBC_AGENT;
		} elseif ($this->agentsDirector->isValidCustomer()) {
			$join = "\n INNER JOIN #__fbchat_session_relations AS rel ON rel.session_id_agent = sess.session_id";
			$join .= "\n INNER JOIN #__fbchat_agents AS ag ON ag.userid = sess.userid";
			$where = "\n AND rel.session_id_customer = " . $this->DBO->Quote($this->session->session_id);
			// Settaggio rango utente
			$this->rango = JFBC_CUSTOMER;
		} else {
			$this->response['buddylist'] = false;
			// Settaggio rango utente
			$this->rango = JFBC_NOUSER;
			return false;
		}
		
		$filter = JFilterInput::getInstance();
		$userFieldName = $filter->clean($this->config->get('usefullname'), 'word)');
		 
		//Prendiamo il time per eventuale aggiornamento lista utenti buddylist
		$time = time();
		
		if ((empty($_SESSION['jfbcchat_buddytime'])) || ($_POST['initialize'] == 1) ||
				(!empty($_SESSION['jfbcchat_buddytime']) && ($time-$_SESSION['jfbcchat_buddytime'] > $this->config->get('buddylistrefresh')))) {
			/*Quello che conta per considerare un utente in stato offline  il tempo dall'ultimo messaggio inviato
			 Al logout completo verr tolta ogni connessione al refresh di pagina*/
			$sql = "SELECT $distinct sess.session_id AS id, u.$userFieldName, sess.time AS lastactivity, ccs.message, ccs.status, ccs.override_name, ccs.skypeid, ccs.skype_enable" . $select .
			"\n FROM #__session AS sess".
			$join .
			"\n LEFT JOIN #__users AS u on sess.userid = u.id" .
			"\n LEFT JOIN #__fbchat_status AS ccs ON sess.session_id = ccs.session_id".
			"\n WHERE sess.session_id <> " . $this->DBO->Quote($this->session->session_id) . " AND sess.client_id = 0" .
			"\n AND sess.time > " . (int)($time-$this->config->get('maxlifetime_session')) .
			"\n AND (ccs.status IS NULL OR ccs.status != 'invisible')" .
			$where .
			"\n ORDER BY u.$userFieldName ASC";
			$this->DBO->setQuery($sql);
			$rows = $this->DBO->loadAssocList();
		
			if(is_array($rows) && count($rows)) {
				foreach ($rows as $chat) {
				// Status utente
					if(is_null($chat['status'])) {
					$chat['status'] = 'available';
					}
					
					// Default message dello status
					if ($chat['message'] == null) {
					$chat['message'] = '';
				}
			
				// Avatar get o generation
				$chat['avatar'] = jfbcAvatar::getAvatar($chat['id']);
			
				// Guest name override: user field name -> override name -> auto generated
				if(!$chat[$userFieldName]) {
					if(!$chat['override_name']) {
						$chat[$userFieldName] = $this->generateRandomGuestNameSuffix($chat['id']);
					} else {
						$chat[$userFieldName] = $chat['override_name'];
					}
				}
					
				// Inject geolocation se esiste per i customers
				if (!isset($chat['geolocation']) || $chat['geolocation'] == null) {
					$chat['geolocation'] = '';
				}
					
				// Inject nowpage se esiste per i customers
				if (!isset($chat['nowpage']) || $chat['nowpage'] == null) {
					$chat['nowpage'] = '';
				}
					
				// Inject skypeid se esiste per gli agents
				if (!isset($chat['skypeid']) || $chat['skypeid'] == null) {
					$chat['skypeid'] = '';
				}
					
				// Inject skype_enable flag se esiste per gli agents
				if (!isset($chat['skype_enable']) || $chat['skype_enable'] == null) {
					$chat['skype_enable'] = 0;
				}
					
				// Array di risposta
				$buddyList[] = array('id' => $chat['id'],
									'name' => $chat[$userFieldName],
									'avatar' => $chat['avatar'],
									'status' => $chat['status'],
									'message' => $chat['message'],
									'nowpage' => $chat['nowpage'],
									'geolocation' => $chat['geolocation'],
									'skypeid' => $chat['skypeid'],
									'skype_enable' => $chat['skype_enable']);
				}
			}
						
					//Riaggiorniamo il time in sessione dell'ultimo refresh lista utenti
			$_SESSION['jfbcchat_buddytime'] = $time;
		
			if (!empty($buddyList)) {
				$this->response['buddylist'] = $buddyList;
				// Iniettiamo anche un array di ID crudo
				if(is_array($buddyList) && count($buddyList)) {
					foreach ($this->response['buddylist'] as $value) {
						$this->response['buddylist_ids'][] = $value['id'];
					}
				}
			} else {
				$this->response['buddylist'] = false;
			}
		}
		return true;
	}
	
	/**
	 * Recupera variabili JS top scope function app
	 * @access protected
	 * @return void
	 */
	protected function getTopScopeJSVars() {
		// Top scope JS side
		// Inject proprio username
		$this->response['amy_username'] = $this->myUser->name;
		if(!$this->myUser->name) {
			// Valutazione eventuale override in base allo status utente logged in/out
			$overrideName = $this->getOverrideName($this->session->session_id);
			if($overrideName) {
				$this->response['amy_username'] = $overrideName;
			} else {
				$this->response['amy_username'] = $this->generateRandomGuestNameSuffix($this->session->session_id);
			}
		}
		// Inject proprio avatar
		$this->response['amy_avatar'] = jfbcAvatar::getAvatar($this->session->session_id);
		// Inject status dell'utente
		$this->response['amy_loggedout'] = !$this->myUser->name;
		// Inject rango dell'utente
		$this->response['amy_rango'] = (int)$this->rango;
	}
	
	/**
	 * Recupera la lista messaggi
	 * @access protected  
	 * @return void
	 */
	protected function fetchMessages() {   
		$openChatBoxId = isset($_SESSION ['jfbcchat_sessionvars'] ['openChatboxId']) ? $_SESSION ['jfbcchat_sessionvars'] ['openChatboxId'] : null ;
		$initialize = JRequest::getVar ( 'initialize' );
		$lastNewMessageID = null;
		$lastReceivedMsgID = JRequest::getInt('last_received_msg_id', 0);
		  
		$sql = "SELECT cchat.id, cchat.session_id_from AS " . $this->DBO->quoteName('from') . ", cchat.session_id_to AS " . $this->DBO->quoteName('to') . 
				"\n , cchat.message, cchat.sent, cchat.read, cchat.type, cchat.status" . 
				"\n FROM #__fbchat AS cchat" .
				"\n LEFT JOIN #__session AS sess ON cchat.session_id_from = sess.session_id" .
				"\n LEFT JOIN #__users AS u ON sess.userid = u.id" .
				"\n WHERE (cchat.session_id_to = ". $this->DBO->Quote($this->session->session_id) . " AND cchat.read != 1)";
		if (!$initialize && $openChatBoxId) {
			$sql .= "\n OR (cchat.session_id_from = ". $this->DBO->Quote($this->session->session_id) . " AND cchat.session_id_to = " . $this->DBO->Quote($openChatBoxId) .
					"\n AND cchat.id > $lastReceivedMsgID AND cchat.type='file')";
		}
		$sql .="\n ORDER BY cchat.id";
	 
		$this->DBO->setQuery($sql);
	 	$rows = $this->DBO->loadAssocList();
	    
	 	if(is_array($rows) && count($rows)) {
		 	// Aggiunta nuovi messaggi
		 	foreach ($rows as $chatmessage) {
		 		$self = 0;
				$old = 0;
				
				// IOC - messaggi da se stessi diventano self 1 e inviati dall'altra parte
				if ($chatmessage['from'] == $this->session->session_id) {
					$chatmessage['from'] = $chatmessage['to'];
					$self = 1;
					$old = 1;
				}  
				$this->messages[] = array( 'id' => $chatmessage['id'],
									 'from' => $chatmessage['from'],  
									 'message' => stripslashes($chatmessage['message']),
									 'type' => @$chatmessage['type'],
									 'status' => @(int)$chatmessage['status'],
									 'self' => $self,
									 'old' => $old);
				
				//Mette i nuovi messaggi provenienti dal mittente in sessione se non propri, vecchi e gi letti
				if ($self == 0 && $old == 0 && $chatmessage['read'] != 1) {
					$_SESSION['jfbcchat_user_'.$chatmessage['from']][] = array('id' => $chatmessage['id'],
																				'from' => $chatmessage['from'],  
																				'message' => stripslashes($chatmessage['message']),
																				'type' => @$chatmessage['type'],
									 											'status' => @(int)$chatmessage['status'],
																				'self' => 0,
																				'old' => 1);
				}
		
				$lastNewMessageID = $chatmessage['id'];
		 	}
	 	}
	 
	 	// Adesso aggiorna lo stato dei messaggi come letti
		if ($lastNewMessageID) {
			$sql = "UPDATE #__fbchat SET `read` = '1' WHERE `session_id_to` = " .
				    $this->DBO->Quote($this->session->session_id) . " and `id` <= " . $this->DBO->Quote($lastNewMessageID); 
				 
			$this->DBO->setQuery($sql); 
			$this->DBO->execute();
		}
		
		// Esplica l'autoresfresh realtime dello status messaggi type=file
		$this->refreshMsgFileSessionStatus();
	}
	  
	/**
	 * Execute della app logic da controller
	 * @access public 
	 * @return void
	 */
	public function appDispatch() {
		$openChatBoxId = null;
		$chatbox = JRequest::getVar ( 'chatbox' );
		$buddylist = JRequest::getVar ( 'buddylist' );
		$initialize = JRequest::getVar ( 'initialize' );
		$update_session = JRequest::getVar ( 'updatesession' );
		$post_sessionvars = JRequest::getVar ( 'sessionvars' );
		
		// Refresh realtime stats con dependency injected object
		$this->realStats->refreshRealtime();
		
		//Inizializzazione variabili di sessione
		if(isset($_SESSION['jfbcchat_user_'.$chatbox])) {
			$fbchat_user_session_messages = $_SESSION['jfbcchat_user_'.$chatbox];
		}
		
		$fbchat_sessionvars = @$_SESSION ['jfbcchat_sessionvars'];
		//$openChatBoxId - l'id dell'utente dell'ultimo box aperto
		if(isset($_SESSION['jfbcchat_sessionvars']['openChatboxId'])) {
			$openChatBoxId = $_SESSION['jfbcchat_sessionvars']['openChatboxId'];
		}
		//$openChatBoxIdUserMessages - un array contentente i messaggi dell'utente dell'ultimo box aperto mantenuti in sessione
		if(isset($_SESSION['jfbcchat_user_'.$openChatBoxId]) && isset($openChatBoxId)) {
			$openChatBoxIdUserMessages = @$_SESSION['jfbcchat_user_'.$openChatBoxId];
		}
		 
		// Si richiede messaggi per un particolare chatbox
		if (! empty ( $chatbox )) {
			if (! empty ( $fbchat_user_session_messages )) {
				$this->messages = $fbchat_user_session_messages;
			}
			// Send and exit
			$this->sendResponse ();
		} else {
			// Se viene richiesta la lista utenti si inserisce nella response
			if (! empty ( $buddylist ) && $buddylist == 1) {
				$this->getBuddyList ();
			}
			 
			// Se  l'initialize = 1 ovvero la prima ajax call si recuperano i messaggi dell'ultimo box aperto dalla sessione
			if (! empty ( $initialize ) && $initialize == 1) {
				// Store server stats con dependency injected object
				$overrideName = $this->myUser->name;
				if(!$overrideName) {
					// Valutazione eventuale override in base allo status utente logged in/out
					$overrideName = $this->getOverrideName($this->session->session_id);
					if(!$overrideName) { 
						$overrideName = $this->generateRandomGuestNameSuffix($this->session->session_id);
					}
				}
				$this->serverStats->storeServerStats($overrideName);
				
				/*
				 * Response populating nel contesto dell'initialize
				 */
				$this->getTopScopeJSVars();
				// Si popolano le proprie status options
				$this->getStatus (); 
				// Si restituiscono le proprie session vars nella variabile initialize alla JS APP
				if (! empty ( $fbchat_sessionvars )) {
					$this->response ['initialize'] = $fbchat_sessionvars;
					if (! empty ( $openChatBoxId ) && ! empty ( $openChatBoxIdUserMessages )) {
						$this->messages = array_merge ( $this->messages, $openChatBoxIdUserMessages );
					}
				} 
				// Inject dei parametri che condizionano la restante parte nella JS APP
				if(!empty($this->config)) { 
					$this->response['aparamslist'] = $this->config->toObject();  
				}
				
				// Inject della suggestion settata come quella di default
				if($suggestion = $this->getDefaultSuggestion()) {
					$this->response['def_suggestion'] = $suggestion;
				}
				
				// Lazy loading initialization objects <<use>>
				// Sezione Garbage dipendenza generica
				if((bool)$this->config->get('gcenabled')){
					$gc = new jfbcGarbage();
					//Exec GC Probability
					$execGC = $gc->execGC();
					//Back to JS domain
					$this->response['execGC'] = $execGC;
				}
				
				// Set start deferred time in sessione
				$_SESSION['jfbcchat_deferredtime'] = time();
				
			} else {
				// Tutte le seguenti ajax call
				if (empty ( $fbchat_sessionvars )) {
					$fbchat_sessionvars = array ();
				}
		
				if (! empty ( $post_sessionvars )) {
					ksort ( $post_sessionvars );
				} else {
					$post_sessionvars = '';
				}
		
				// Settaggio in sessione delle session_vars in post dalla JS APP tra cui activeChatBoxes con iduser|nummessagesnew
				if (! empty ( $update_session ) && $update_session == 1) {
					$_SESSION ['jfbcchat_sessionvars'] = array_merge($fbchat_sessionvars, $post_sessionvars);
				}
				
				// Deferred in cui  incluso anche l'assignAgent ad intervalli e l'inject delle top scope my vars
				if (isset($_SESSION['jfbcchat_deferredtime']) && (time()-$_SESSION['jfbcchat_deferredtime'] > $this->config->get('deferredrefresh'))) {
					// Esecuzione algoritmo assegnamento customers->agents
					$this->agentsDirector->assignAgent();
					$this->getTopScopeJSVars();
					// Refresh time
					$_SESSION['jfbcchat_deferredtime'] = time();
				}
			}
		
			//Otteniamo la lista messaggi con gestione in dependency injection
			$this->fetchMessages ();
			$this->sendResponse ();
		}
	}
	
	/**
	 * Class constructor
	 * @access public 
	 * @param Object& $session 
	 * @param Object& $agentsDirector
	 * @param Object& $realStats
	 * @param Object& $serverStats
	 * @return Object&
	 */
	public function __construct(&$session, &$agentsDirector, &$realStats, &$serverStats) {
		// init
		$this->messages = array();
		$this->response = array();
		$this->session = &$session;
		$this->agentsDirector = &$agentsDirector;
		$this->realStats = &$realStats;
		$this->serverStats = &$serverStats;
		$this->myUser = JFactory::getUser();
		$this->DBO = JFactory::getDBO();
		$this->config = JComponentHelper::getParams ( 'com_jfbchat' );
		
		$this->queryChunk = null;
		$this->customerGroups = $this->config->get('customer_gids');
		// Query chunk settings
		if(is_array($this->customerGroups)) {
			if(in_array(1, $this->customerGroups)) {
				$this->queryChunk = " OR ISNULL(map.group_id)";
			}
		}
	}
}

